diff -pruN 4.3.2-1/.github/actions/docker-init/action.yml 4.4-1/.github/actions/docker-init/action.yml
--- 4.3.2-1/.github/actions/docker-init/action.yml	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/.github/actions/docker-init/action.yml	2024-05-31 16:23:43.000000000 +0000
@@ -33,9 +33,12 @@ runs:
 
     - name: Determine Debian tag
       run: |
+        COMMIT_DATE=$(date -d @$(git log -1 --pretty=%ct) +%Y%m%d)
         DEBIAN_RELEASE=$(grep -m 1 'ARG DEBIAN_TAG=' Dockerfile | sed 's/.*DEBIAN_TAG=\(.*\)-.*/\1/')
-        echo "DEBIAN_TAG=$(podman image search --list-tags debian --limit 1000000000 | \
-                           grep "$DEBIAN_RELEASE-.*-slim" | sort -r | head -1 | sed 's/.*[ ]\+//')" >> $GITHUB_ENV
+        echo "DEBIAN_TAG=$(podman search --list-tags docker.io/debian --limit 1000000000 | \
+                           grep "$DEBIAN_RELEASE-.*-slim" | sort -r | sed 's/.*[ ]\+//' | \
+                           ./scripts/lower-bound.py $DEBIAN_RELEASE-$COMMIT_DATE-slim )" \
+                           >> $GITHUB_ENV
       shell: bash
 
     - name: Prepare repository for COPY-in
@@ -62,33 +65,28 @@ runs:
         annotations: |
           org.opencontainers.image.description=${{ env.IMAGE_DESCRIPTION }}
           org.opencontainers.image.licenses=MIT and others
+          # redact information that is not reproducible
+          org.opencontainers.image.created=
+          org.opencontainers.image.source=
+          org.opencontainers.image.url=
       env:
         DOCKER_METADATA_ANNOTATIONS_LEVELS: manifest,index
 
-    - name: Cache var-cache-apt
-      id: cache-var-cache-apt
+    - name: Cache apt
+      id: cache-apt
       uses: actions/cache@v4
       with:
-        path: var-cache-apt
-        key: var-cache-apt-${{ env.DEBIAN_TAG }}-${{ inputs.image-name }}
+        path: |
+          var-cache-apt
+          var-lib-apt
+        key: cache-apt-${{ env.DEBIAN_TAG }}-${{ inputs.image-name }}
 
-    - name: Cache var-lib-apt
-      id: cache-var-lib-apt
-      uses: actions/cache@v4
-      with:
-        path: var-lib-apt
-        key: var-lib-apt-${{ env.DEBIAN_TAG }}-${{ inputs.image-name }}
-
-    - name: Inject var-cache-apt into docker
-      uses: reproducible-containers/buildkit-cache-dance@v2.1.4
-      with:
-        cache-source: var-cache-apt
-        cache-target: /var/cache/apt
-        skip-extraction: ${{ steps.cache-var-cache-apt.outputs.cache-hit }}
-
-    - name: Inject var-lib-apt into docker
-      uses: reproducible-containers/buildkit-cache-dance@v2.1.4
+    - name: Inject cache into docker
+      uses: reproducible-containers/buildkit-cache-dance@v3.1.0
       with:
-        cache-source: var-lib-apt
-        cache-target: /var/lib/apt
-        skip-extraction: ${{ steps.cache-var-lib-apt.outputs.cache-hit }}
+        cache-map: |
+          {
+            "var-cache-apt": "/var/cache/apt",
+            "var-lib-apt": "/var/lib/apt"
+          }
+        skip-extraction: ${{ steps.cache.outputs.cache-hit }}
diff -pruN 4.3.2-1/.github/workflows/master.yml 4.4-1/.github/workflows/master.yml
--- 4.3.2-1/.github/workflows/master.yml	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/.github/workflows/master.yml	2024-05-31 16:23:43.000000000 +0000
@@ -9,6 +9,11 @@ jobs:
   deploy_containers:
     name: Build and deploy container images
     runs-on: ubuntu-latest
+    permissions:
+      id-token: write
+      packages: write
+      contents: read
+      attestations: write
     strategy:
       matrix:
         image-name: ["kas", "kas-isar"]
@@ -35,6 +40,7 @@ jobs:
       - name: Build ${{ matrix.image-name }} image
         uses: docker/build-push-action@v5
         if: ${{ env.PUSH_MASTER == 'true' }}
+        id: push
         with:
           context: /home/runner/kas-clone
           target: ${{ matrix.image-name }}
@@ -42,7 +48,13 @@ jobs:
           build-args: |
             SOURCE_DATE_EPOCH=${{ env.SOURCE_DATE_EPOCH }}
             DEBIAN_TAG=${{ env.DEBIAN_TAG }}
-          provenance: mode=max,reproducible=true
+          provenance: false
           outputs: type=registry,rewrite-timestamp=true
-          tags: ghcr.io/siemens/kas/${{ matrix.image-name }}
+          tags: ghcr.io/${{ github.repository }}/${{ matrix.image-name }}
           annotations: ${{ env.DOCKER_METADATA_OUTPUT_ANNOTATIONS }}
+      - name: Attest ${{ matrix.image-name }} image
+        uses: actions/attest-build-provenance@v1
+        with:
+          subject-name: ghcr.io/${{ github.repository }}/${{ matrix.image-name }}
+          subject-digest: ${{ steps.push.outputs.digest }}
+          push-to-registry: true
diff -pruN 4.3.2-1/.github/workflows/next.yml 4.4-1/.github/workflows/next.yml
--- 4.3.2-1/.github/workflows/next.yml	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/.github/workflows/next.yml	2024-05-31 16:23:43.000000000 +0000
@@ -65,6 +65,11 @@ jobs:
     name: Build, test and deploy container images
     needs: perform_tests
     runs-on: ubuntu-latest
+    permissions:
+      id-token: write
+      packages: write
+      contents: read
+      attestations: write
     strategy:
       matrix:
         image-name: ["kas", "kas-isar"]
@@ -88,13 +93,16 @@ jobs:
             SOURCE_DATE_EPOCH=${{ env.SOURCE_DATE_EPOCH }}
             DEBIAN_TAG=${{ env.DEBIAN_TAG }}
           outputs: type=docker,rewrite-timestamp=true
-          tags: ghcr.io/siemens/kas/${{ matrix.image-name }}:next
+          tags: ghcr.io/${{ github.repository }}/${{ matrix.image-name }}:next
       - name: Test ${{ matrix.image-name }} image
         run: |
           cd image-tests/${{ matrix.image-name }}
-          KAS_IMAGE_VERSION=next ../../kas-container build kas.yml
+          KAS_CONTAINER_IMAGE=ghcr.io/${{ github.repository }}/${{ matrix.image-name }}:next \
+          KAS_CLONE_DEPTH=1 \
+          ../../kas-container build kas.yml
       - name: Complete build and deploy ${{ matrix.image-name }} image
         uses: docker/build-push-action@v5
+        id: push
         with:
           context: /home/runner/kas-clone
           target: ${{ matrix.image-name }}
@@ -102,7 +110,13 @@ jobs:
           build-args: |
             SOURCE_DATE_EPOCH=${{ env.SOURCE_DATE_EPOCH }}
             DEBIAN_TAG=${{ env.DEBIAN_TAG }}
-          provenance: mode=max,reproducible=true
+          provenance: false
           outputs: type=registry,rewrite-timestamp=true
-          tags: ghcr.io/siemens/kas/${{ matrix.image-name }}:next
+          tags: ghcr.io/${{ github.repository }}/${{ matrix.image-name }}:next
           annotations: ${{ env.DOCKER_METADATA_OUTPUT_ANNOTATIONS }}
+      - name: Attest ${{ matrix.image-name }} image
+        uses: actions/attest-build-provenance@v1
+        with:
+          subject-name: ghcr.io/${{ github.repository }}/${{ matrix.image-name }}
+          subject-digest: ${{ steps.push.outputs.digest }}
+          push-to-registry: true
diff -pruN 4.3.2-1/.github/workflows/release.yml 4.4-1/.github/workflows/release.yml
--- 4.3.2-1/.github/workflows/release.yml	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/.github/workflows/release.yml	2024-05-31 16:23:43.000000000 +0000
@@ -9,6 +9,11 @@ jobs:
   deploy_containers:
     name: Build and deploy container images
     runs-on: ubuntu-latest
+    permissions:
+      id-token: write
+      packages: write
+      contents: read
+      attestations: write
     strategy:
       matrix:
         image-name: ["kas", "kas-isar"]
@@ -25,6 +30,7 @@ jobs:
           image-name: ${{ matrix.image-name }}
       - name: Build ${{ matrix.image-name }} image
         uses: docker/build-push-action@v5
+        id: push
         with:
           context: /home/runner/kas-clone
           target: ${{ matrix.image-name }}
@@ -32,10 +38,16 @@ jobs:
           build-args: |
             SOURCE_DATE_EPOCH=${{ env.SOURCE_DATE_EPOCH }}
             DEBIAN_TAG=${{ env.DEBIAN_TAG }}
-          provenance: mode=max,reproducible=true
+          provenance: false
           outputs: type=registry,rewrite-timestamp=true
           tags: |
-            ghcr.io/siemens/kas/${{ matrix.image-name }}
-            ghcr.io/siemens/kas/${{ matrix.image-name }}:${{ env.RELEASE_VERSION }}
-            ghcr.io/siemens/kas/${{ matrix.image-name }}:latest-release
+            ghcr.io/${{ github.repository }}/${{ matrix.image-name }}
+            ghcr.io/${{ github.repository }}/${{ matrix.image-name }}:${{ env.RELEASE_VERSION }}
+            ghcr.io/${{ github.repository }}/${{ matrix.image-name }}:latest-release
           annotations: ${{ env.DOCKER_METADATA_OUTPUT_ANNOTATIONS }}
+      - name: Attest ${{ matrix.image-name }} image
+        uses: actions/attest-build-provenance@v1
+        with:
+          subject-name: ghcr.io/${{ github.repository }}/${{ matrix.image-name }}
+          subject-digest: ${{ steps.push.outputs.digest }}
+          push-to-registry: true
diff -pruN 4.3.2-1/CHANGELOG.md 4.4-1/CHANGELOG.md
--- 4.3.2-1/CHANGELOG.md	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/CHANGELOG.md	2024-05-31 16:23:43.000000000 +0000
@@ -1,3 +1,18 @@
+4.4
+- kas: Auto-import runner-provided .gitconfig also inside GitLab-CI
+- kas: Auto-inject git credentials on gitlab ci
+- kas: Add --keep-config-unchanged to preserve repos and configs on actions
+- kas: Fix --skip'ing multiple steps
+- kas: List --skip'able steps in --help
+- kas: Add support for shallow clones
+- kas: Add support to create provenance build attestations
+- kas: Add config key to describe build artifacts (used by attestation)
+- kas: Add option to dump-plugin to include VCS info of local repos
+- kas-container: Handle missing extra argument in subcommands gracefully
+- kas-container: improve container reproduction using git commit date
+- docs: Several format improvements
+- docs: Add simple examples
+
 4.3.2
 - kas: don't add comments to .netrc, fixing gitlab-ci
 - kas: make file permissions on credentials more strict (not a security fix)
diff -pruN 4.3.2-1/debian/changelog 4.4-1/debian/changelog
--- 4.3.2-1/debian/changelog	2024-04-10 04:49:59.000000000 +0000
+++ 4.4-1/debian/changelog	2024-06-06 09:40:53.000000000 +0000
@@ -1,3 +1,11 @@
+kas (4.4-1) unstable; urgency=medium
+
+  * debian: update upstream signing key
+  * New upstream version 4.4
+  * debian: add attestation man page
+
+ -- Felix Moessbauer <felix.moessbauer@siemens.com>  Thu, 06 Jun 2024 11:40:53 +0200
+
 kas (4.3.2-1) unstable; urgency=medium
 
   * New upstream version 4.3.2
diff -pruN 4.3.2-1/debian/kas.manpages 4.4-1/debian/kas.manpages
--- 4.3.2-1/debian/kas.manpages	2024-04-10 04:49:59.000000000 +0000
+++ 4.4-1/debian/kas.manpages	2024-06-06 09:38:35.000000000 +0000
@@ -1,5 +1,6 @@
 docs/_build/man/kas.1
 docs/_build/man/kas-build.1
+docs/_build/man/kas-build-attestation.1
 docs/_build/man/kas-checkout.1
 docs/_build/man/kas-credentials.1
 docs/_build/man/kas-dump.1
diff -pruN 4.3.2-1/debian/upstream/signing-key.asc 4.4-1/debian/upstream/signing-key.asc
--- 4.3.2-1/debian/upstream/signing-key.asc	2024-04-10 04:49:59.000000000 +0000
+++ 4.4-1/debian/upstream/signing-key.asc	2024-06-06 09:25:55.000000000 +0000
@@ -1,31 +1,88 @@
 -----BEGIN PGP PUBLIC KEY BLOCK-----
 
-mQGhBEq0i8QRBAD2wOxlC9m/8t/vqjm1U9yQCT6OJ2Wbv/qys9DYM0CvcOTDMWQw
-mV1/VsZjKR5YgB5NPt+To7X6x5cjz15AGnx5Fb8Wnrq8EF9ZfHMwb7YMx1LdPYPD
-nXr37wE3XupFmkHBMes4htyX7Dz8LvKDLnli4IsAmWG+kV1JI6LWKXLpSwCg8JRq
-4SWoB4VCQxbH3GjntgvwNc8D/2q5Dj0urJ7s7sdXhrH0hcFkpSFRmU5Yd6MCwcbF
-Hm2paL5gqVjNNUUwDBKAL9eZaQVIHKwC44BvNGO2gcQ26R3AuCHUQ+pZHg34tok1
-JCNZ6IEZccb+33Qq0qbcDMJJiDYp7ppp6ozifvc4YaqJECX48IydxfE9+41oV7T5
-vNAzA/QL/UMJyTnu5jiOXcyn5iFQw535lXkwKsqeXzCowLhoHICZ2jITJPdTT/+9
-pGWwMQqST/SS35Tx4EnS3z2BWsNMCLuXCPkxF1elaMJqMfMJxFD8rAgS9GK6zP6f
-JlsA1wq/UvKSL8v4QPOnTNCVOsyqJVasGV0ZPcDfcj+ClNO4tB5KYW4gS2lzemth
-IDxqYW4ua2lzemthQHdlYi5kZT6IYwQTEQIAIwIbIwYLCQgHAwIEFQIIAwQWAgMB
-Ah4BAheABQJOpoNtAhkBAAoJEIrUrG965ecUOPUAoK+Rh12KgCjplHAS0AoiGKwG
-Ouq7AKDEVnBtRAoyVRvp3lOlOx+P2Ay56LQjSmFuIEtpc3prYSA8amFuLmtpc3pr
-YUBzaWVtZW5zLmNvbT6IYgQTEQIAIgUCTqaDSAIbIwYLCQgHAwIGFQgCCQoLBBYC
-AwECHgECF4AACgkQitSsb3rl5xSP9QCdFgfK84fWExUKeuVDF1N2eVJjkPAAnA1W
-hzAzJzktDFIXgV2pR8PEVO3duQINBEq0i8QQCADb8+VfqjKLt1OgfQIkIerR4qZl
-prsKYZpWfHJ56kq8h+IzyfxN3voBsbrh+NEQzCBKlvgX9TK0XfP/Te2Ph7j1b/Mq
-NsFEHb/CYVGVkEL1yuDbZD5uC8w6zGcLq4ddpY3x8v6NV6L/k6gc6gWpoyxfwS+m
-WiK2Uj8E+P5WBbbqOSx/dhnKTWGdyWxn8hiFfOBGZUmjhGJNb9gtACTCd3Bz58o9
-r8hgsQNbUlziQ0ugrQ/3RvjnOO7SGmXQLTSlU5FM63MUr0bkdruo4BYl8BUk5Eco
-CSS/B+1/eGSk90vK9lRZEde9R7GYDrV8u8MEAWFHFgYPy3mDPzxIXlPOr7TLAAMF
-B/9hKRpisSyU9aIIDbBToRbTJqoFAChsWw4y+y58nqjae2rk1yrftUjcoJPrt4c8
-T3H7M9wQ2hlaxGsL1L08J/Us6yLvJtmxHXcAmoMWVHMlniSWH1DCDAVnFQHM50i+
-23DMbVCTMfHTpEpmW4Vik8XcNF25Iy6R0gnbCZYJT6iEH6Zh597rkkaNhB8dWzya
-BhUU4/FtF5r1UkAIIy6ymhlamPjdR1HWUsvIhQVjP6Opw9bsd38ZMjDg3ruliXR3
-A8OYV3iiEnmDhmmrrW2g3KwEYS4SbkYJyfRWYa1ClJidHlxWblvbI/Phsi4U5K5l
-Sj+Xu+K11jSLLT/C8psEL40riEkEGBECAAkFAkq0i8QCGwwACgkQitSsb3rl5xQu
-8ACgoa/GzlPciHJBPwjywyzIC8ITjAcAnjpLXrJpv47E2S856g4RfQScdUdo
-=bIsJ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+=2O7e
 -----END PGP PUBLIC KEY BLOCK-----
diff -pruN 4.3.2-1/docs/_man/_kas-man-footer.inc 4.4-1/docs/_man/_kas-man-footer.inc
--- 4.3.2-1/docs/_man/_kas-man-footer.inc	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/_man/_kas-man-footer.inc	2024-05-31 16:23:43.000000000 +0000
@@ -1,4 +1,4 @@
 KAS
-~~~
+---
 
 Part of the :manpage:`kas(1)` suite.
diff -pruN 4.3.2-1/docs/_man/kas-build-attestation.rst 4.4-1/docs/_man/kas-build-attestation.rst
--- 4.3.2-1/docs/_man/kas-build-attestation.rst	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/docs/_man/kas-build-attestation.rst	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,10 @@
+:orphan:
+
+kas build attestation
+=====================
+
+.. toctree::
+
+    ../userguide/build-attestation
+
+.. include:: _kas-man-footer.inc
diff -pruN 4.3.2-1/docs/_man/kas-credentials.rst 4.4-1/docs/_man/kas-credentials.rst
--- 4.3.2-1/docs/_man/kas-credentials.rst	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/_man/kas-credentials.rst	2024-05-31 16:23:43.000000000 +0000
@@ -3,6 +3,8 @@
 kas credential handling
 =======================
 
-.. include:: ../userguide/credentials.inc
+.. toctree::
+
+   ../userguide/credentials
 
 .. include:: _kas-man-footer.inc
diff -pruN 4.3.2-1/docs/_man/kas-format-changelog.rst 4.4-1/docs/_man/kas-format-changelog.rst
--- 4.3.2-1/docs/_man/kas-format-changelog.rst	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/_man/kas-format-changelog.rst	2024-05-31 16:23:43.000000000 +0000
@@ -3,6 +3,9 @@
 Configuration Format Changes
 ============================
 
+DESCRIPTION
+-----------
+
 The kas project configuration is versioned to ensure that the requested
 features are supported by the current kas version.
 
@@ -12,7 +15,7 @@ features are supported by the current ka
    ../format-changelog
 
 SEE ALSO
-~~~~~~~~
+--------
 
 :manpage:`kas-project-config(1)`
 
diff -pruN 4.3.2-1/docs/_man/kas-plugin-build.rst 4.4-1/docs/_man/kas-plugin-build.rst
--- 4.3.2-1/docs/_man/kas-plugin-build.rst	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/_man/kas-plugin-build.rst	2024-05-31 16:23:43.000000000 +0000
@@ -3,6 +3,19 @@
 kas build plugin
 ================
 
-.. include:: ../userguide/plugins/build.inc
+.. argparse::
+    :module: kas.kas
+    :func: kas_get_argparser
+    :prog: kas
+    :path: build
+    :manpage:
+
+    .. automodule:: kas.plugins.build
+        :noindex:
+
+SEE ALSO
+~~~~~~~~
+
+:manpage:`kas-build-attestation(1)`,
 
 .. include:: _kas-man-footer.inc
diff -pruN 4.3.2-1/docs/_man/kas-plugin-checkout.rst 4.4-1/docs/_man/kas-plugin-checkout.rst
--- 4.3.2-1/docs/_man/kas-plugin-checkout.rst	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/_man/kas-plugin-checkout.rst	2024-05-31 16:23:43.000000000 +0000
@@ -3,6 +3,14 @@
 kas checkout plugin
 ===================
 
-.. include:: ../userguide/plugins/checkout.inc
+.. argparse::
+    :module: kas.kas
+    :func: kas_get_argparser
+    :prog: kas
+    :path: checkout
+    :manpage:
+
+    .. automodule:: kas.plugins.checkout
+        :noindex:
 
 .. include:: _kas-man-footer.inc
diff -pruN 4.3.2-1/docs/_man/kas-plugin-dump.rst 4.4-1/docs/_man/kas-plugin-dump.rst
--- 4.3.2-1/docs/_man/kas-plugin-dump.rst	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/_man/kas-plugin-dump.rst	2024-05-31 16:23:43.000000000 +0000
@@ -3,10 +3,18 @@
 kas dump plugin
 ===============
 
-.. include:: ../userguide/plugins/dump.inc
+.. argparse::
+    :module: kas.kas
+    :func: kas_get_argparser
+    :prog: kas
+    :path: dump
+    :manpage:
+
+    .. automodule:: kas.plugins.dump
+        :noindex:
 
 SEE ALSO
-~~~~~~~~
+--------
 
 :manpage:`kas-project-config(1)`,
 :manpage:`kas-checkout(1)`,
diff -pruN 4.3.2-1/docs/_man/kas-plugin-for-all-repos.rst 4.4-1/docs/_man/kas-plugin-for-all-repos.rst
--- 4.3.2-1/docs/_man/kas-plugin-for-all-repos.rst	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/_man/kas-plugin-for-all-repos.rst	2024-05-31 16:23:43.000000000 +0000
@@ -3,6 +3,14 @@
 kas for-all-repos plugin
 ========================
 
-.. include:: ../userguide/plugins/for-all-repos.inc
+.. argparse::
+    :module: kas.kas
+    :func: kas_get_argparser
+    :prog: kas
+    :path: for-all-repos
+    :manpage:
+
+    .. automodule:: kas.plugins.for_all_repos
+        :noindex:
 
 .. include:: _kas-man-footer.inc
diff -pruN 4.3.2-1/docs/_man/kas-plugin-menu.rst 4.4-1/docs/_man/kas-plugin-menu.rst
--- 4.3.2-1/docs/_man/kas-plugin-menu.rst	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/_man/kas-plugin-menu.rst	2024-05-31 16:23:43.000000000 +0000
@@ -3,6 +3,14 @@
 kas menu plugin
 ===============
 
-.. include:: ../userguide/plugins/menu.inc
+.. argparse::
+    :module: kas.kas
+    :func: kas_get_argparser
+    :prog: kas
+    :path: menu
+    :manpage:
+
+    .. automodule:: kas.plugins.menu
+        :noindex:
 
 .. include:: _kas-man-footer.inc
diff -pruN 4.3.2-1/docs/_man/kas-plugin-shell.rst 4.4-1/docs/_man/kas-plugin-shell.rst
--- 4.3.2-1/docs/_man/kas-plugin-shell.rst	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/_man/kas-plugin-shell.rst	2024-05-31 16:23:43.000000000 +0000
@@ -3,6 +3,14 @@
 kas shell plugin
 ================
 
-.. include:: ../userguide/plugins/shell.inc
+.. argparse::
+    :module: kas.kas
+    :func: kas_get_argparser
+    :prog: kas
+    :path: shell
+    :manpage:
+
+    .. automodule:: kas.plugins.shell
+        :noindex:
 
 .. include:: _kas-man-footer.inc
diff -pruN 4.3.2-1/docs/_man/kas-project-config.rst 4.4-1/docs/_man/kas-project-config.rst
--- 4.3.2-1/docs/_man/kas-project-config.rst	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/_man/kas-project-config.rst	2024-05-31 16:23:43.000000000 +0000
@@ -3,10 +3,12 @@
 kas project configuration
 =========================
 
-.. include:: ../userguide/project-configuration.inc
+  .. toctree::
+
+     ../userguide/project-configuration
 
 SEE ALSO
-~~~~~~~~
+--------
 
 :manpage:`kas-format-changelog(1)`
 
diff -pruN 4.3.2-1/docs/_man/kas.rst 4.4-1/docs/_man/kas.rst
--- 4.3.2-1/docs/_man/kas.rst	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/_man/kas.rst	2024-05-31 16:23:43.000000000 +0000
@@ -1,17 +1,43 @@
 :orphan:
 
-Welcome to the kas documentation, a setup tool for bitbake based projects
-=========================================================================
+kas manpage
+===========
 
-.. toctree::
-   :maxdepth: 2
+.. argparse::
+    :module: kas.kas
+    :func: kas_get_argparser
+    :prog: kas
+    :manpage:
 
-   ../intro
-   ../userguide
-   ../command-line
+    .. include:: ../intro.rst
+        :start-line: 3
+
+PROJECT CONFIGURATION
+---------------------
+
+The project configuration file describes the build environment and the layers
+to be used. It is the main input to kas.
+For details, see :manpage:`kas-project-config(1)`
+
+BUILD ATTESTATION
+-----------------
+
+Kas supports to generate build attestation. For details, see
+:manpage:`kas-build-attestation(1)`.
+
+CREDENTIAL HANDLING
+-------------------
+
+kas provides various mechanisms to inject credentials into the build.
+For details, see :manpage:`kas-credentials(1)`.
+
+ENVIRONMENT VARIABLES
+---------------------
+
+.. include:: ../command-line/environment-variables.inc
 
 SEE ALSO
-~~~~~~~~
+--------
 
 :manpage:`kas-project-config(1)`,
 :manpage:`kas-build(1)`,
diff -pruN 4.3.2-1/docs/command-line/environment-variables.inc 4.4-1/docs/command-line/environment-variables.inc
--- 4.3.2-1/docs/command-line/environment-variables.inc	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/docs/command-line/environment-variables.inc	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,162 @@
+kas uses a number of environment variables to configure its behavior.
+The `Variables Glossary`_ provides an overview, wherein the tuple (C,K,E)
+denotes the scope of the variable.
+
+Variable Scope
+~~~~~~~~~~~~~~
+
+**kas-container (C)**
+
+The variable is processed or forwarded by the ``kas-container`` script.
+For some variables, the variable is re-written to the container's
+directory layout.
+
+.. note::
+    The ``env`` section of the `project configuration` can be used to make
+    arbitrary environment variables available to the build environment. When
+    invoking the build via ``kas-container``, make sure to also forward the
+    corresponding environment variables into the container.
+
+**kas (K)**
+
+The variable is processed by kas itself. Some variables (e.g. the credentials
+for the awscli) are re-written to configuration files to also support older
+versions of the tooling.
+
+**build environment (E)**
+
+The variable is exported into the build environment. In this environment, the
+``bitbake`` command is executed.
+
+
+Variables Glossary
+~~~~~~~~~~~~~~~~~~
+
++--------------------------+--------------------------------------------------+
+| Environment variables    | Description                                      |
++==========================+==================================================+
+| ``KAS_WORK_DIR``         | The path of the kas work directory, current work |
+| (C, K)                   | directory is the default.                        |
++--------------------------+--------------------------------------------------+
+| ``KAS_BUILD_DIR``        | The path build directory,                        |
+| (C, K)                   | ``${KAS_WORK_DIR}/build`` is the default.        |
++--------------------------+--------------------------------------------------+
+| ``KAS_REPO_REF_DIR``     | The path to the repository reference directory.  |
+| (C, K)                   | Repositories in this directory are used as       |
+|                          | references when cloning. In order for kas to     |
+|                          | find those repositories, they have to be named   |
+|                          | in a specific way. The repo URLs are translated  |
+|                          | like this:                                       |
+|                          | "https://github.com/siemens/meta-iot2000.git"    |
+|                          | resolves to the name                             |
+|                          | "github.com.siemens.meta-iot2000.git".           |
+|                          | Repositories that are not found will be cloned   |
+|                          | below this directory. Multiple instances of kas  |
+|                          | can simultaneously work on the same directory,   |
+|                          | as long as the underlying filesystem is POSIX    |
+|                          | compatible.                                      |
++--------------------------+--------------------------------------------------+
+| ``KAS_DISTRO``           | This overwrites the respective setting in the    |
+| ``KAS_MACHINE``          | configuration file.                              |
+| ``KAS_TARGET``           |                                                  |
+| ``KAS_TASK``             |                                                  |
+| (C, K)                   |                                                  |
++--------------------------+--------------------------------------------------+
+| ``KAS_PREMIRRORS``       | Specifies alternatives for repo URLs. Just like  |
+| (C, K)                   | bitbake ``PREMIRRORS``, this variable consists   |
+|                          | of new-line separated entries. Each entry        |
+|                          | defines a regular expression to match a URL and, |
+|                          | space-separated, its replacement. E.g.:          |
+|                          | "http://.*\.someurl\.io/ http://localmirror.net/"|
++--------------------------+--------------------------------------------------+
+| ``DISTRO_APT_PREMIRRORS``| Specifies alternatives for apt URLs. Just like   |
+| (C)                      | ``KAS_PREMIRRORS``.                              |
++--------------------------+--------------------------------------------------+
+| ``KAS_CLONE_DEPTH``      | Perform shallow git clone/fetch using --depth=N  |
+| (C, K)                   | specified by this variable. This is useful in    |
+|                          | case CI always starts with empty work directory  |
+|                          | and this directory is always discarded after the |
+|                          | CI run.                                          |
++--------------------------+--------------------------------------------------+
+| ``SSH_PRIVATE_KEY``      | Variable containing the private key that should  |
+| (K)                      | be added to an internal ssh-agent. This key      |
+|                          | cannot be password protected. This setting is    |
+|                          | useful for CI build servers. On desktop          |
+|                          | machines, an ssh-agent running outside the kas   |
+|                          | environment is more useful.                      |
++--------------------------+--------------------------------------------------+
+| ``SSH_PRIVATE_KEY_FILE`` | Path to the private key file that should be      |
+| (K)                      | added to an internal ssh-agent. This key cannot  |
+|                          | be password protected. This setting is useful    |
+|                          | for CI build servers. On desktop machines, an    |
+|                          | ssh-agent running outside the kas environment is |
+|                          | more useful.                                     |
++--------------------------+--------------------------------------------------+
+| ``SSH_AUTH_SOCK``        | SSH authentication socket. Used for cloning over |
+| (C,K,E)                  | SSH (alternative to ``SSH_PRIVATE_KEY`` or       |
+|                          | ``SSH_PRIVATE_KEY_FILE``).                       |
++--------------------------+--------------------------------------------------+
+| ``DL_DIR``               | Environment variables that are transferred to    |
+| ``SSTATE_DIR``           | the bitbake environment.                         |
+| ``SSTATE_MIRRORS``       |                                                  |
+| (C,K,E)                  |                                                  |
++--------------------------+--------------------------------------------------+
+| ``TMPDIR`` (K,E)         | Directory for temporary files.                   |
++--------------------------+--------------------------------------------------+
+| ``http_proxy``           | These variables define the proxy configuration   |
+| ``https_proxy``          | bitbake should use.                              |
+| ``ftp_proxy``            |                                                  |
+| ``no_proxy``             |                                                  |
+| (C,K,E)                  |                                                  |
++--------------------------+--------------------------------------------------+
+| ``GIT_PROXY_COMMAND`` (E)| Set proxy for native git fetches. ``NO_PROXY``   |
+| ``NO_PROXY`` (C,K,E)     | is evaluated by OpenEmbedded's oe-git-proxy      |
+|                          | script.                                          |
++--------------------------+--------------------------------------------------+
+| ``SHELL``                | The shell to start when using the `shell`        |
+| (C,K,E)                  | plugin.                                          |
++--------------------------+--------------------------------------------------+
+| ``TERM``                 | The terminal options used in the `shell` plugin. |
+| (C,K,E)                  |                                                  |
++--------------------------+--------------------------------------------------+
+| ``AWS_CONFIG_FILE``      | Path to the awscli configuration and credentials |
+| |aws_cred|               | files that are copied to the kas home dir.       |
+| (K,C)                    |                                                  |
++--------------------------+--------------------------------------------------+
+| |git_cred|               | Allows to set and configure the git credential   |
+| (K,C)                    | helper in the `.gitconfig` of the kas user.      |
++--------------------------+--------------------------------------------------+
+| ``GITCONFIG_FILE``       | Path to a `.gitconfig` file which will be        |
+| (K,C)                    | copied to the kas home dir as `.gitconfig`.      |
++--------------------------+--------------------------------------------------+
+| ``NETRC_FILE``           | Path to a .netrc file which will be copied to    |
+| (K,C)                    | the kas home dir as .netrc.                      |
++--------------------------+--------------------------------------------------+
+| ``CI_SERVER_HOST``       | Environment variables from GitLab CI, if set     |
+| ``CI_JOB_TOKEN``         | .netrc is configured to allow fetching from      |
+| ``CI_JOB_URL``           | the GitLab instance. An entry will be appended   |
+| (K)                      | in case ``NETRC_FILE`` was given as well. Note   |
+|                          | that if the file already contains an entry for   |
+|                          | that host most tools would probably take that    |
+|                          | first one. The job url is added to the           |
+|                          | provenance attestation (if enabled).             |
++--------------------------+--------------------------------------------------+
+| ``GITHUB_ACTIONS``       | Environment variables from GitHub actions or     |
+| ``GITLAB_CI``            | GitLab CI. If set to `true`, `.gitconfig` is     |
+| (K)                      | automatically imported.                          |
+|                          | For details, see ``GITCONFIG_FILE``.             |
++--------------------------+--------------------------------------------------+
+| ``BB_NUMBER_THREADS``    | Environment variables to control the concurrency.|
+| ``PARALLEL_MAKE``        |                                                  |
+| (C,K,E)                  |                                                  |
++--------------------------+--------------------------------------------------+
+
+.. |aws_cred| replace:: ``AWS_ROLE_ARN``
+                        ``AWS_SHARED_CREDENTIALS_FILE``
+                        ``AWS_WEB_IDENTITY_TOKEN_FILE``
+.. |git_cred| replace:: ``GIT_CREDENTIAL_HELPER`` ``GIT_CREDENTIAL_USEHTTPPATH``
+
+.. only:: html
+
+    For details about the access of remote resources, see
+    :ref:`checkout-creds-label`.
diff -pruN 4.3.2-1/docs/command-line.rst 4.4-1/docs/command-line.rst
--- 4.3.2-1/docs/command-line.rst	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/command-line.rst	2024-05-31 16:23:43.000000000 +0000
@@ -7,161 +7,9 @@ Command Line Usage
     :prog: kas
     :nosubcommands:
 
-
 .. _env-vars-label:
 
 Environment Variables
 ---------------------
 
-kas uses a number of environment variables to configure its behavior.
-The `Variables Glossary`_ provides an overview, wherein the tuple (C,K,E)
-denotes the scope of the variable.
-
-Variable Scope
-~~~~~~~~~~~~~~
-
-**kas-container (C)**
-
-The variable is processed or forwarded by the ``kas-container`` script.
-For some variables, the variable is re-written to the container's
-directory layout.
-
-.. note::
-    The ``env`` section of the `project configuration` can be used to make
-    arbitrary environment variables available to the build environment. When
-    invoking the build via ``kas-container``, make sure to also forward the
-    corresponding environment variables into the container.
-
-**kas (K)**
-
-The variable is processed by kas itself. Some variables (e.g. the credentials
-for the awscli) are re-written to configuration files to also support older
-versions of the tooling.
-
-**build environment (E)**
-
-The variable is exported into the build environment. In this environment, the
-``bitbake`` command is executed.
-
-
-Variables Glossary
-~~~~~~~~~~~~~~~~~~
-
-+--------------------------+--------------------------------------------------+
-| Environment variables    | Description                                      |
-+==========================+==================================================+
-| ``KAS_WORK_DIR``         | The path of the kas work directory, current work |
-| (C, K)                   | directory is the default.                        |
-+--------------------------+--------------------------------------------------+
-| ``KAS_BUILD_DIR``        | The path build directory,                        |
-| (C, K)                   | ``${KAS_WORK_DIR}/build`` is the default.        |
-+--------------------------+--------------------------------------------------+
-| ``KAS_REPO_REF_DIR``     | The path to the repository reference directory.  |
-| (C, K)                   | Repositories in this directory are used as       |
-|                          | references when cloning. In order for kas to     |
-|                          | find those repositories, they have to be named   |
-|                          | in a specific way. The repo URLs are translated  |
-|                          | like this:                                       |
-|                          | "https://github.com/siemens/meta-iot2000.git"    |
-|                          | resolves to the name                             |
-|                          | "github.com.siemens.meta-iot2000.git".           |
-|                          | Repositories that are not found will be cloned   |
-|                          | below this directory. Multiple instances of kas  |
-|                          | can simultaneously work on the same directory,   |
-|                          | as long as the underlying filesystem is POSIX    |
-|                          | compatible.                                      |
-+--------------------------+--------------------------------------------------+
-| ``KAS_DISTRO``           | This overwrites the respective setting in the    |
-| ``KAS_MACHINE``          | configuration file.                              |
-| ``KAS_TARGET``           |                                                  |
-| ``KAS_TASK``             |                                                  |
-| (C, K)                   |                                                  |
-+--------------------------+--------------------------------------------------+
-| ``KAS_PREMIRRORS``       | Specifies alternatives for repo URLs. Just like  |
-| (C, K)                   | bitbake ``PREMIRRORS``, this variable consists   |
-|                          | of new-line separated entries. Each entry        |
-|                          | defines a regular expression to match a URL and, |
-|                          | space-separated, its replacement. E.g.:          |
-|                          | "http://.*\.someurl\.io/ http://localmirror.net/"|
-+--------------------------+--------------------------------------------------+
-| ``DISTRO_APT_PREMIRRORS``| Specifies alternatives for apt URLs. Just like   |
-| (C)                      | ``KAS_PREMIRRORS``.                              |
-+--------------------------+--------------------------------------------------+
-| ``SSH_PRIVATE_KEY``      | Variable containing the private key that should  |
-| (K)                      | be added to an internal ssh-agent. This key      |
-|                          | cannot be password protected. This setting is    |
-|                          | useful for CI build servers. On desktop          |
-|                          | machines, an ssh-agent running outside the kas   |
-|                          | environment is more useful.                      |
-+--------------------------+--------------------------------------------------+
-| ``SSH_PRIVATE_KEY_FILE`` | Path to the private key file that should be      |
-| (K)                      | added to an internal ssh-agent. This key cannot  |
-|                          | be password protected. This setting is useful    |
-|                          | for CI build servers. On desktop machines, an    |
-|                          | ssh-agent running outside the kas environment is |
-|                          | more useful.                                     |
-+--------------------------+--------------------------------------------------+
-| ``SSH_AUTH_SOCK``        | SSH authentication socket. Used for cloning over |
-| (C,K,E)                  | SSH (alternative to ``SSH_PRIVATE_KEY`` or       |
-|                          | ``SSH_PRIVATE_KEY_FILE``).                       |
-+--------------------------+--------------------------------------------------+
-| ``DL_DIR``               | Environment variables that are transferred to    |
-| ``SSTATE_DIR``           | the bitbake environment.                         |
-| ``SSTATE_MIRRORS``       |                                                  |
-| (C,K,E)                  |                                                  |
-+--------------------------+--------------------------------------------------+
-| ``TMPDIR`` (K,E)         | Directory for temporary files.                   |
-+--------------------------+--------------------------------------------------+
-| ``http_proxy``           | These variables define the proxy configuration   |
-| ``https_proxy``          | bitbake should use.                              |
-| ``ftp_proxy``            |                                                  |
-| ``no_proxy``             |                                                  |
-| (C,K,E)                  |                                                  |
-+--------------------------+--------------------------------------------------+
-| ``GIT_PROXY_COMMAND`` (E)| Set proxy for native git fetches. ``NO_PROXY``   |
-| ``NO_PROXY`` (C,K,E)     | is evaluated by OpenEmbedded's oe-git-proxy      |
-|                          | script.                                          |
-+--------------------------+--------------------------------------------------+
-| ``SHELL``                | The shell to start when using the `shell`        |
-| (C,K,E)                  | plugin.                                          |
-+--------------------------+--------------------------------------------------+
-| ``TERM``                 | The terminal options used in the `shell` plugin. |
-| (C,K,E)                  |                                                  |
-+--------------------------+--------------------------------------------------+
-| ``AWS_CONFIG_FILE``      | Path to the awscli configuration and credentials |
-| |aws_cred|               | files that are copied to the kas home dir.       |
-| (K,C)                    |                                                  |
-+--------------------------+--------------------------------------------------+
-| |git_cred|               | Allows to set and configure the git credential   |
-| (K,C)                    | helper in the `.gitconfig` of the kas user.      |
-+--------------------------+--------------------------------------------------+
-| ``GITCONFIG_FILE``       | Path to a `.gitconfig` file which will be        |
-| (K,C)                    | copied to the kas home dir as `.gitconfig`.      |
-+--------------------------+--------------------------------------------------+
-| ``NETRC_FILE``           | Path to a .netrc file which will be copied to    |
-| (K,C)                    | the kas home dir as .netrc.                      |
-+--------------------------+--------------------------------------------------+
-| ``CI_SERVER_HOST``       | Environment variables from gitlab CI, if set     |
-| ``CI_JOB_TOKEN``         | .netrc is configured to allow fetching from      |
-| (K)                      | the gitlab instance. An entry will be appended   |
-|                          | in case ``NETRC_FILE`` was given as well. Note   |
-|                          | that if the file already contains an entry for   |
-|                          | that host most tools would probably take that    |
-|                          | first one.                                       |
-+--------------------------+--------------------------------------------------+
-| ``GITHUB_ACTIONS``       | Environment variables from github actions. If    |
-| (K)                      | set to `true`, `.gitconfig` is automatically     |
-|                          | imported. For details, see ``GITCONFIG_FILE``.   |
-+--------------------------+--------------------------------------------------+
-| ``BB_NUMBER_THREADS``    | Environment variables to control the concurrency.|
-| ``PARALLEL_MAKE``        |                                                  |
-| (C,K,E)                  |                                                  |
-+--------------------------+--------------------------------------------------+
-
-.. |aws_cred| replace:: ``AWS_ROLE_ARN``
-                        ``AWS_SHARED_CREDENTIALS_FILE``
-                        ``AWS_WEB_IDENTITY_TOKEN_FILE``
-.. |git_cred| replace:: ``GIT_CREDENTIAL_HELPER`` ``GIT_CREDENTIAL_USEHTTPPATH``
-
-For details about the access of remote resources, see
-:ref:`checkout-creds-label`.
+.. include:: command-line/environment-variables.inc
diff -pruN 4.3.2-1/docs/conf.py 4.4-1/docs/conf.py
--- 4.3.2-1/docs/conf.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/conf.py	2024-05-31 16:23:43.000000000 +0000
@@ -343,6 +343,9 @@ man_pages = [
      [author], 1),
     ('_man/kas-credentials', 'kas-credentials', 'kas credential handling',
      [author], 1),
+    ('_man/kas-build-attestation',
+     'kas-build-attestation', 'working with SLSA / in-toto attestations',
+     [author], 1),
 ]
 
 # If true, show URL addresses after external links.
diff -pruN 4.3.2-1/docs/format-changelog.rst 4.4-1/docs/format-changelog.rst
--- 4.3.2-1/docs/format-changelog.rst	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/format-changelog.rst	2024-05-31 16:23:43.000000000 +0000
@@ -163,3 +163,11 @@ Fixed
 
 - The key ``tag`` introduced in v15 was not supported in ``defaults``.
   It's now added.
+
+Version 17
+----------
+
+Added
+~~~~~
+
+- The key ``artifacts`` is introduced to describe expected build artifacts.
diff -pruN 4.3.2-1/docs/userguide/build-attestation.rst 4.4-1/docs/userguide/build-attestation.rst
--- 4.3.2-1/docs/userguide/build-attestation.rst	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/docs/userguide/build-attestation.rst	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,106 @@
+Build Attestation
+=================
+
+kas supports the generation of SLSA / in-toto build attestation data.
+Currently, support for the following attestation formats is implemented:
+
+- `provenance v1 <https://slsa.dev/spec/v1.0/provenance>`_
+
+Provenance
+----------
+
+The provenance data provides information about the build. This includes
+data about the build environment, as well as all primary (first-level)
+dependencies. The attestation will be stored in
+``attestation/kas-build.provenance.json``
+
+The following modes are supported: ``mode=min``, ``mode=max``, whereby
+the mode controls the amount of information that is included in the
+attestation. The CLI options hereby loosely follow the ``docker buildx``
+provenance options, which are described in detail in
+`Provenance attestations <https://docs.docker.com/build/attestations/slsa-provenance/>`_.
+For compatibility with the ``docker buildx`` CLI options, we also support
+``--provenance true``, which is equivalent to ``--provenance mode=min``.
+
+In min mode, the provenance attestation will contain information about:
+
+- Build timestamps
+- Build materials (project-config files)
+- Source repositories and revisions
+- Build platform
+
+In max mode, it will also contain the build environment, i.e. all
+environment variables of the ``env`` section along with their values.
+
+.. warning::
+    In max mode, the provenance attestation captures all environment
+    variables specified in the ``env`` section. Make sure to not expose
+    any secrets.
+
+For example, to build the configuration described in the file
+``kas-project.yml`` and emit provenance attestations, you could run::
+
+    kas build --provenance mode=max kas-project.yml
+
+Working with sigstore cosign
+----------------------------
+
+The `cosign tool <https://github.com/sigstore/cosign>`_ from the `sigstore
+project <https://www.sigstore.dev/>`_ (`documentation <https://docs.sigstore.dev/>`_)
+has native support for in-toto build predicates. However, it currently can only
+operate directly on the predicate but not on the enclosing attestation
+(cosign 2.2.4). By that, the predicate first needs to be extracted (provenance
+in this example)::
+
+    cat build/attestation/kas-build.provenance.json | jq '.predicate' > provenance.json
+
+Attestation Signing
+~~~~~~~~~~~~~~~~~~~
+
+.. warning::
+    The following command operates on the public append-only transparency
+    log. Make sure to understand the implications before executing.
+
+The following example shows how to create a signed build attestation
+from a provenance predicate and a local artifact, using the OIDC keyless
+signing workflow:
+
+.. code-block:: sh
+
+    cosign attest-blob \
+        --type=slsaprovenance1 \
+        --predicate=provenance.json \
+        --output-signature=kas-build.dsse.json \
+        --output-certificate=cert.pem \
+        <image>.wic
+
+.. note::
+    Currently the attestation can only be created for a single artifact file.
+
+For later verification, both the signed attestation (``kas-build.dsse.json``),
+as well as the certificate (``cert.pem``) are needed. Make sure to
+ship them along with the artifact.
+
+Attestation Verification
+~~~~~~~~~~~~~~~~~~~~~~~~
+
+The previously signed blob can be verified again:
+
+.. code-block:: sh
+
+    cosign verify-blob-attestation \
+        --certificate=cert.pem \
+        --signature=kas-build.dsse.json \
+        --certificate-identity="<redacted>" \
+        --certificate-oidc-issuer="<oidc issuer>" \
+        --type=slsaprovenance1 \
+        <image>.wic
+
+Attestation Restoration
+~~~~~~~~~~~~~~~~~~~~~~~
+
+The dsse record (``kas-build.dsse.json``) contains the original
+attestation data as base64 encoded string. To restore the attestation,
+run::
+
+    cat kas-build.dsse.json | jq -r '.payload' | base64 -d | jq
diff -pruN 4.3.2-1/docs/userguide/credentials.inc 4.4-1/docs/userguide/credentials.inc
--- 4.3.2-1/docs/userguide/credentials.inc	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/userguide/credentials.inc	1970-01-01 00:00:00.000000000 +0000
@@ -1,60 +0,0 @@
-kas provides various mechanisms to inject credentials into the build.
-By using :ref:`env-vars-label`, a fine grained control is possible. All
-credentials are made available both to KAS, as well as inside the build
-environment. However, not all mechanisms are natively supported by all tools.
-As kas might need to modify credentials and config files, these are copied
-into the isolated environment first. One exception is the SSH folder, where
-changes are only performed if not yet present on the host.
-
-.. only:: man
-
-  For details about credential related environment variables,
-  see :manpage:`kas(1)`.
-
-AWS Configuration
-~~~~~~~~~~~~~~~~~
-
-For AWS, both conventional AWS config files as well as the environment
-variable controlled OAuth 2.0 workflow are supported. Note, that KAS
-internally rewrites the ``AWS_*`` environment variables into a AWS
-config file to also support older versions of bitbake.
-
-Git Configuration
-~~~~~~~~~~~~~~~~~
-
-A ``.gitconfig`` file can be used to provide credentials as well as
-url rewrites of git repositories (``insteadof``). To support the patching
-of git repositories, kas injects a ``[user]`` section, possibly overwriting
-an existing one. When running in the Github CI, the ``.gitconfig`` file is
-automatically injected. In addition, credential helpers can be used by
-setting the corresponding environment variables. These are added to the
-``.gitconfig`` file as well.
-
-Netrc File
-~~~~~~~~~~
-
-A ``.netrc`` file can be used to provide credentials for git or the
-HTTP(S) / FTP fetcher. When running in the Gitlab CI, the ``CI_JOB_TOKEN``
-is appended to automatically grant access to repositories that can be
-accessed by the user that triggered the CI pipeline.
-
-SSH
-~~~
-
-The ssh folder of the calling user is automatically shared with kas. This
-is currently not controllable, as ssh does not obey the ``$HOME`` variable.
-This can be used to inject both credentials, as well as ssh configuration
-items into the kas environment.
-
-.. note::
-  Modifications to the ``.ssh/config`` file are only performed if the file
-  is not present yet.
-
-In addition, an external ssh-agent can be made available in the kas
-environment by setting the ``SSH_AUTH_SOCK`` environment variable.
-As an alternative, ssh private keys can be added to an internal ssh agent
-by setting ``SSH_PRIVATE_KEY`` or ``SSH_PRIVATE_KEY_FILE``.
-
-.. note::
-  The use of an external ssh agent cannot be combined with options that
-  require an internal ssh agent.
diff -pruN 4.3.2-1/docs/userguide/credentials.rst 4.4-1/docs/userguide/credentials.rst
--- 4.3.2-1/docs/userguide/credentials.rst	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/docs/userguide/credentials.rst	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,92 @@
+.. _checkout-creds-label:
+
+Credential Handling
+===================
+
+kas provides various mechanisms to inject credentials into the build.
+By using :ref:`env-vars-label`, a fine grained control is possible. All
+credentials are made available both to KAS, as well as inside the build
+environment. However, not all mechanisms are natively supported by all tools.
+As kas might need to modify credentials and config files, these are copied
+into the isolated environment first. One exception is the SSH folder, where
+changes are only performed if not yet present on the host.
+
+.. only:: man
+
+  For details about credential related environment variables,
+  see :manpage:`kas(1)`.
+
+AWS Configuration
+-----------------
+
+For AWS, both conventional AWS config files as well as the environment
+variable controlled OAuth 2.0 workflow are supported. Note, that KAS
+internally rewrites the ``AWS_*`` environment variables into a AWS
+config file to also support older versions of bitbake.
+
+Git Configuration
+-----------------
+
+A ``.gitconfig`` file can be used to provide credentials as well as
+url rewrites of git repositories (``insteadof``). To support the patching
+of git repositories, kas injects a ``[user]`` section, possibly overwriting
+an existing one. In addition, credential helpers can be used by
+setting the corresponding environment variables. These are added to the
+``.gitconfig`` file as well.
+
+When running in a GitHub Action or GitLab CI job, the ``.gitconfig`` file
+is automatically injected.
+
+GitHub Actions
+~~~~~~~~~~~~~~
+
+In combination with the
+`webfactory/ssh-agent <https://github.com/webfactory/ssh-agent>`_ action,
+this automatically makes the required credentials available to kas and
+bitbake.
+
+GitLab CI
+~~~~~~~~~
+
+When running in the GitLab CI, the ``CI_JOB_TOKEN`` can be used to access
+git repositories via https. kas automatically adds this token to the
+``.netrc`` file, where it is picked up by git. Further, kas configures git
+to automatically rewrite the urls of the repositories to clone via https
+for repos stored on the same server. Technically this is achieved by adding
+`insteadof` entries to the ``.gitconfig`` file.
+
+For backwards compatibility, the git rewrite rules are only added if
+``.gitconfig`` does not exists and no SSH configuration is provided.
+
+.. note::
+  Make sure to assign the correct permissions to the ``CI_JOB_TOKEN``.
+  For details, see `GitLab CI/CD job token <https://docs.gitlab.com/ee/ci/jobs/ci_job_token.html>`_.
+
+Netrc File
+----------
+
+A ``.netrc`` file can be used to provide credentials for git or the
+HTTP(S) / FTP fetcher. When running in the GitLab CI, the ``CI_JOB_TOKEN``
+is appended to automatically grant access to repositories that can be
+accessed by the user that triggered the CI pipeline.
+
+SSH
+---
+
+The ssh folder of the calling user is automatically shared with kas. This
+is currently not controllable, as ssh does not obey the ``$HOME`` variable.
+This can be used to inject both credentials, as well as ssh configuration
+items into the kas environment.
+
+.. note::
+  Modifications to the ``.ssh/config`` file are only performed if the file
+  is not present yet.
+
+In addition, an external ssh-agent can be made available in the kas
+environment by setting the ``SSH_AUTH_SOCK`` environment variable.
+As an alternative, ssh private keys can be added to an internal ssh agent
+by setting ``SSH_PRIVATE_KEY`` or ``SSH_PRIVATE_KEY_FILE``.
+
+.. note::
+  The use of an external ssh agent cannot be combined with options that
+  require an internal ssh agent.
diff -pruN 4.3.2-1/docs/userguide/getting-started.rst 4.4-1/docs/userguide/getting-started.rst
--- 4.3.2-1/docs/userguide/getting-started.rst	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/docs/userguide/getting-started.rst	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,78 @@
+Getting Started
+===============
+
+Dependencies & installation
+---------------------------
+
+This project depends on
+
+- Python 3
+- distro Python 3 package
+- jsonschema Python 3 package
+- PyYAML Python 3 package
+- GitPython Python 3 package
+- kconfiglib Python 3 package (optional, for menu plugin)
+- NEWT Python 3 distro package (optional, for menu plugin)
+
+To install kas into your python site-package repository, run::
+
+    $ sudo pip3 install .
+
+Usage
+-----
+
+There are (at least) four options for using kas:
+
+- Install it locally via pip to get the ``kas`` command.
+- Use the container image locally. In this case, download the ``kas-container``
+  script from the kas repository and use it in place of the ``kas`` command.
+  The script version corresponds to the kas tool and the kas image version.
+- Use the container image in CI. Specify
+  ``ghcr.io/siemens/kas/kas[-isar][:<x.y>]`` in your CI script that requests
+  a container image as runtime environment. See
+  https://github.com/orgs/siemens/packages/container/kas%2Fkas/versions and
+  https://github.com/orgs/siemens/packages/container/kas%2Fkas-isar/versions for
+  all available images.
+- Use the **run-kas** wrapper from this directory. In this case,
+  replace ``kas`` in the examples below with ``path/to/run-kas``.
+
+Start build::
+
+    $ kas build /path/to/kas-project.yml
+
+Alternatively, experienced bitbake users can invoke usual **bitbake** steps
+manually, e.g.::
+
+    $ kas shell /path/to/kas-project.yml -c 'bitbake dosfsutils-native'
+
+For details about the kas input file(s), see
+:ref:`project-configuration-label`. Example configurations can be found in
+:ref:`example-configurations-label`.
+
+.. note::
+  kas will place downloads and build artifacts under the current directory when
+  being invoked. You can specify a different location via the environment
+  variable ``KAS_WORK_DIR``.
+
+
+Use Cases
+---------
+
+1.  Initial build/setup::
+
+    $ mkdir $PROJECT_DIR
+    $ cd $PROJECT_DIR
+    $ git clone $PROJECT_URL meta-project
+    $ kas build meta-project/kas-project.yml
+
+2.  Update/rebuild::
+
+    $ cd $PROJECT_DIR/meta-project
+    $ git pull
+    $ kas build kas-project.yml
+
+3.  Interactive configuration::
+
+    $ cd $PROJECT_DIR/meta-project
+    $ kas menu
+    $ kas build  # optional, if not triggered via kas menu
diff -pruN 4.3.2-1/docs/userguide/plugins/build.inc 4.4-1/docs/userguide/plugins/build.inc
--- 4.3.2-1/docs/userguide/plugins/build.inc	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/userguide/plugins/build.inc	1970-01-01 00:00:00.000000000 +0000
@@ -1,8 +0,0 @@
-.. automodule:: kas.plugins.build
-    :noindex:
-
-.. argparse::
-    :module: kas.kas
-    :func: kas_get_argparser
-    :prog: kas
-    :path: build
diff -pruN 4.3.2-1/docs/userguide/plugins/checkout.inc 4.4-1/docs/userguide/plugins/checkout.inc
--- 4.3.2-1/docs/userguide/plugins/checkout.inc	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/userguide/plugins/checkout.inc	1970-01-01 00:00:00.000000000 +0000
@@ -1,8 +0,0 @@
-.. automodule:: kas.plugins.checkout
-    :noindex:
-
-.. argparse::
-    :module: kas.kas
-    :func: kas_get_argparser
-    :prog: kas
-    :path: checkout
diff -pruN 4.3.2-1/docs/userguide/plugins/dump.inc 4.4-1/docs/userguide/plugins/dump.inc
--- 4.3.2-1/docs/userguide/plugins/dump.inc	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/userguide/plugins/dump.inc	1970-01-01 00:00:00.000000000 +0000
@@ -1,8 +0,0 @@
-.. automodule:: kas.plugins.dump
-    :noindex:
-
-.. argparse::
-    :module: kas.kas
-    :func: kas_get_argparser
-    :prog: kas
-    :path: dump
diff -pruN 4.3.2-1/docs/userguide/plugins/for-all-repos.inc 4.4-1/docs/userguide/plugins/for-all-repos.inc
--- 4.3.2-1/docs/userguide/plugins/for-all-repos.inc	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/userguide/plugins/for-all-repos.inc	1970-01-01 00:00:00.000000000 +0000
@@ -1,8 +0,0 @@
-.. automodule:: kas.plugins.for_all_repos
-    :noindex:
-
-.. argparse::
-    :module: kas.kas
-    :func: kas_get_argparser
-    :prog: kas
-    :path: for-all-repos
diff -pruN 4.3.2-1/docs/userguide/plugins/menu.inc 4.4-1/docs/userguide/plugins/menu.inc
--- 4.3.2-1/docs/userguide/plugins/menu.inc	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/userguide/plugins/menu.inc	1970-01-01 00:00:00.000000000 +0000
@@ -1,8 +0,0 @@
-.. automodule:: kas.plugins.menu
-    :noindex:
-
-.. argparse::
-    :module: kas.kas
-    :func: kas_get_argparser
-    :prog: kas
-    :path: menu
diff -pruN 4.3.2-1/docs/userguide/plugins/shell.inc 4.4-1/docs/userguide/plugins/shell.inc
--- 4.3.2-1/docs/userguide/plugins/shell.inc	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/userguide/plugins/shell.inc	1970-01-01 00:00:00.000000000 +0000
@@ -1,8 +0,0 @@
-.. automodule:: kas.plugins.shell
-    :noindex:
-
-.. argparse::
-    :module: kas.kas
-    :func: kas_get_argparser
-    :prog: kas
-    :path: shell
diff -pruN 4.3.2-1/docs/userguide/plugins.rst 4.4-1/docs/userguide/plugins.rst
--- 4.3.2-1/docs/userguide/plugins.rst	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/docs/userguide/plugins.rst	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,72 @@
+Sub-commands (Plugins)
+======================
+
+kas sub-commands are implemented by a series of plugins. Each plugin
+typically provides a single command.
+
+``build`` plugin
+----------------
+
+.. automodule:: kas.plugins.build
+
+.. argparse::
+    :module: kas.kas
+    :func: kas_get_argparser
+    :prog: kas
+    :path: build
+
+``checkout`` plugin
+-------------------
+
+.. automodule:: kas.plugins.checkout
+
+.. argparse::
+    :module: kas.kas
+    :func: kas_get_argparser
+    :prog: kas
+    :path: checkout
+
+
+``dump`` plugin
+---------------
+
+.. automodule:: kas.plugins.dump
+
+.. argparse::
+    :module: kas.kas
+    :func: kas_get_argparser
+    :prog: kas
+    :path: dump
+
+``for-all-repos`` plugin
+------------------------
+
+.. automodule:: kas.plugins.for_all_repos
+
+.. argparse::
+    :module: kas.kas
+    :func: kas_get_argparser
+    :prog: kas
+    :path: for-all-repos
+
+``menu`` plugin
+---------------
+
+.. automodule:: kas.plugins.menu
+
+.. argparse::
+    :module: kas.kas
+    :func: kas_get_argparser
+    :prog: kas
+    :path: menu
+
+``shell`` plugin
+----------------
+
+.. automodule:: kas.plugins.shell
+
+.. argparse::
+    :module: kas.kas
+    :func: kas_get_argparser
+    :prog: kas
+    :path: shell
diff -pruN 4.3.2-1/docs/userguide/project-configuration.inc 4.4-1/docs/userguide/project-configuration.inc
--- 4.3.2-1/docs/userguide/project-configuration.inc	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/userguide/project-configuration.inc	1970-01-01 00:00:00.000000000 +0000
@@ -1,452 +0,0 @@
-Currently, JSON and YAML are supported as the base file formats. Since YAML is
-arguably easier to read, this documentation focuses on the YAML format.
-
-.. code-block:: yaml
-
-    # Every file needs to contain a header, that provides kas with information
-    # about the context of this file.
-    header:
-      # The `version` entry in the header describes for which configuration
-      # format version this file was created for. It is used by kas to figure
-      # out if it is compatible with this file. The version is an integer that
-      # is increased on every format change.
-      version: x
-    # The machine as it is written into the `local.conf` of bitbake.
-    machine: qemux86-64
-    # The distro name as it is written into the `local.conf` of bitbake.
-    distro: poky
-    repos:
-      # This entry includes the repository where the config file is located
-      # to the bblayers.conf:
-      meta-custom:
-      # Here we include a list of layers from the poky repository to the
-      # bblayers.conf:
-      poky:
-        url: "https://git.yoctoproject.org/git/poky"
-        commit: 89e6c98d92887913cadf06b2adb97f26cde4849b
-        layers:
-          meta:
-          meta-poky:
-          meta-yocto-bsp:
-
-A minimal input file consists out of the ``header``, ``machine``, ``distro``,
-and ``repos``.
-
-Additionally, you can add ``bblayers_conf_header`` and ``local_conf_header``
-which are strings that are added to the head of the respective files
-(``bblayers.conf`` or ``local.conf``):
-
-.. code-block:: yaml
-
-    bblayers_conf_header:
-      meta-custom: |
-        POKY_BBLAYERS_CONF_VERSION = "2"
-        BBPATH = "${TOPDIR}"
-        BBFILES ?= ""
-    local_conf_header:
-      meta-custom: |
-        PATCHRESOLVE = "noop"
-        CONF_VERSION = "1"
-        IMAGE_FSTYPES = "tar"
-
-``meta-custom`` in these examples should be a unique name for this
-configuration entries.
-
-We recommend that this unique name is the **same** as the name of the
-containing repository/layer to ease cross-project referencing.
-
-In given examples we assume that your configuration file is part of a
-``meta-custom`` repository/layer. This way it is possible to overwrite or
-append entries in files that include this configuration by naming an entry
-the same (overwriting) or using an unused name (appending).
-
-Including in-tree configuration files
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
-It's currently possible to include kas configuration files from the same
-repository/layer like this:
-
-.. code-block:: yaml
-
-    header:
-      version: x
-      includes:
-        - base.yml
-        - bsp.yml
-        - product.yml
-
-The paths to the files in the include list are either absolute, if they start
-with a `/`, or relative.
-
-If the path is relative and the configuration file is inside a repository,
-then path is relative to the repositories base directory. If the
-configuration file is not in a repository, then the path is relative to the
-parent directory of the file.
-
-Including configuration files from other repos
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
-It's also possible to include configuration files from other repos like this:
-
-.. code-block:: yaml
-
-    header:
-      version: x
-      includes:
-        - repo: poky
-          file: kas-poky.yml
-        - repo: meta-bsp-collection
-          file: hw1/kas-hw-bsp1.yml
-        - repo: meta-custom
-          file: products/product.yml
-    repos:
-      meta-custom:
-      meta-bsp-collection:
-        url: "https://www.example.com/git/meta-bsp-collection"
-        commit: 3f786850e387550fdab836ed7e6dc881de23001b
-        layers:
-          # Additional to the layers that are added from this repository
-          # in the hw1/kas-hw-bsp1.yml, we add here an additional bsp
-          # meta layer:
-          meta-custom-bsp:
-      poky:
-        url: "https://git.yoctoproject.org/git/poky"
-        commit: 89e6c98d92887913cadf06b2adb97f26cde4849b
-        layers:
-          # If `kas-poky.yml` adds the `meta-yocto-bsp` layer and we
-          # do not want it in our bblayers for this project, we can
-          # overwrite it by setting:
-          meta-yocto-bsp: excluded
-
-The files are addressed relative to the git repository path.
-
-The include mechanism collects and merges the content from top to bottom and
-depth first. That means that settings in one include file are overwritten
-by settings in a latter include file and entries from the last include file can
-be overwritten by the current file. While merging all the dictionaries are
-merged recursively while preserving the order in which the entries are added to
-the dictionary. This means that ``local_conf_header`` entries are added to the
-``local.conf`` file in the same order in which they are defined in the
-different include files. Note that the order of the configuration file entries
-is not preserved within one include file, because the parser creates normal
-unordered dictionaries.
-
-Including configuration files via the command line
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
-When specifying the kas configuration file on the command line, additional
-configurations can be included ad-hoc:
-
-    $ kas build kas-base.yml:debug-image.yml:board.yml
-
-This is equivalent to static inclusion from some kas-combined.yml like this:
-
-.. code-block:: yaml
-
-    header:
-      version: x
-      includes:
-        - kas-base.yml
-        - debug.image.yml
-        - board.yml
-
-Command line inclusion allows to create configurations on-demand, without the
-need to write a kas configuration file for each possible combination.
-
-Note that all configuration files combined via the command line either have to
-come from the same repository or have to live outside of any versioning control.
-kas will refuse any other combination in order to avoid complications and
-configuration flaws that can easily emerge from them.
-
-Working with lockfiles
-~~~~~~~~~~~~~~~~~~~~~~
-
-kas supports the use of lockfiles to pinpoint repositories to exact commit ID
-(e.g. SHA-1 refs for git). A lockfile hereby only overrides the commit ID
-defined in a kas file. When performing the checkout operation (or any other
-operation that performs a checkout), kas checks if a file named
-``<filename>.lock.<ext>`` is found next to the currently processed kas file.
-If this is found, kas loads this file right after processing the current one.
-Note, that this applies to both files on the kas cmdline, as well as included
-files.
-
-The following example shows this mechanism for a file ``kas/kas-isar.yml``
-and its corresponding lockfile ``kas/kas-isar.lock.yml``.
-
-``kas/kas-isar.yml``:
-
-.. code-block:: yaml
-
-  # [...]
-  repos:
-    isar:
-      url: https://github.com/ilbers/isar.git
-      branch: next
-
-``kas/kas-isar.lock.yml``:
-
-.. code-block:: yaml
-
-  header:
-    version: 14
-  overrides:
-    repos:
-      isar:
-        commit: 0336610df8bb0adce76ef8c5a921c758efed9f45
-
-The ``dump`` plugin provides helpers to simplify the creation and update
-of lockfiles. For details, see the plugins documentation: :mod:`kas.plugins.dump`.
-
-Configuration reference
-~~~~~~~~~~~~~~~~~~~~~~~
-
-``header``: dict [required]
-  The header of every kas configuration file. It contains information about
-  the context of the file.
-
-  ``version``: integer [required]
-    Lets kas check if it is compatible with this file. See the
-    :doc:`configuration format changelog <../format-changelog>` for the
-    format history and the latest available version.
-
-  ``includes``: list [optional]
-    A list of configuration files this current file is based on. They are
-    merged in order they are stated. So a latter one could overwrite
-    settings from previous files. The current file can overwrite settings
-    from every included file. An item in this list can have one of two types:
-
-    item: string
-      The path to a kas configuration file, relative to the repository root
-      of the current file.
-
-    item: dict
-      If files from other repositories should be included, choose this
-      representation.
-
-      ``repo``: string [required]
-        The id of the repository where the file is located. The repo
-        needs to be defined in the ``repos`` dictionary as ``<repo-id>``.
-
-      ``file``: string [required]
-        The path to the file, relative to the root of the specified
-        repository.
-
-``build_system``: string [optional]
-  Defines the bitbake-based build system. Known build systems are
-  ``openembedded`` (or ``oe``) and ``isar``. If set, this restricts the
-  search of kas for the init script in the configured repositories to
-  ``oe-init-build-env`` or ``isar-init-build-env``, respectively. If
-  ``kas-container`` finds this property in the top-level kas configuration
-  file (includes are not evaluated), it will automatically select the
-  required container image and invocation mode.
-
-``defaults``: dict [optional]
-  This key can be used to set default values for various properties.
-  This may help you to avoid repeating the same property assignment in
-  multiple places if, for example, you wish to use the same branch for
-  all repositories.
-
-  ``repos``: dict [optional]
-    This key can contain default values for some repository properties.
-    If a default value is set for a repository property it may still be
-    overridden by setting the same property to a different value in a given
-    repository.
-
-    ``branch``: string [optional]
-      Sets the default ``branch`` property applied to all repositories that
-      do not override this.
-
-    ``tag``: string [optional]
-      Sets the default ``tag`` property applied to all repositories that
-      do not override this.
-
-    ``patches``: dict [optional]
-      This key can contain default values for some repository patch
-      properties. If a default value is set for a patch property it may
-      still be overridden by setting the same property to a different value
-      in a given patch.
-
-      ``repo``: string [optional]
-        Sets the default ``repo`` property applied to all repository
-        patches that do not override this.
-
-``machine``: string [optional]
-  Contains the value of the ``MACHINE`` variable that is written into the
-  ``local.conf``. Can be overwritten by the ``KAS_MACHINE`` environment
-  variable and defaults to ``qemux86-64``.
-
-``distro``: string [optional]
-  Contains the value of the ``DISTRO`` variable that is written into the
-  ``local.conf``. Can be overwritten by the ``KAS_DISTRO`` environment
-  variable and defaults to ``poky``.
-
-``target``: string [optional] or list [optional]
-  Contains the target or a list of targets to build by bitbake. Can be
-  overwritten by the ``KAS_TARGET`` environment variable and defaults to
-  ``core-image-minimal``. Space is used as a delimiter if multiple targets
-  should be specified via the environment variable.
-  For targets prefixed with ``multiconfig:`` or ``mc:``, corresponding
-  entries are added to the ``BBMULTICONFIG`` in ``local.conf``.
-
-``env``: dict [optional]
-  Contains environment variable names with either default values or ``null``.
-  These variables are made available to bitbake via
-  ``BB_ENV_PASSTHROUGH_ADDITIONS`` (``BB_ENV_EXTRAWHITE`` in older Bitbake
-  versions) and can be overwritten by the variables of the environment in
-  which kas is started.
-  Either a string or nothing (``null``) can be assigned as value.
-  The former one serves as a default value whereas the latter one will lead
-  to add the variable only to ``BB_ENV_PASSTHROUGH_ADDITIONS`` and not to
-  the environment where kas is started. Please note, that ``null`` needs to
-  be assigned as the nulltype (e.g. ``MYVAR: null``), not as 'null'.
-
-``task``: string [optional]
-  Contains the task to build by bitbake. Can be overwritten by the
-  ``KAS_TASK`` environment variable and defaults to ``build``.
-
-``repos``: dict [optional]
-  Contains the definitions of all available repos and layers.
-
-  ``<repo-id>``: dict [optional]
-    Contains the definition of a repository and the layers, that should be
-    part of the build. If the value is ``None``, the repository, where the
-    current configuration file is located is defined as ``<repo-id>`` and
-    added as a layer to the build. It is recommended that the ``<repo-id>``
-    is related to the containing repository/layer to ease cross-project
-    referencing.
-
-    ``name``: string [optional]
-      Defines under which name the repository is stored. If its missing
-      the ``<repo-id>`` will be used.
-
-    ``url``: string [optional]
-      The url of the repository. If this is missing, no version control
-      operations are performed.
-
-    ``type``: string [optional]
-      The type of version control repository. The default value is ``git``
-      and ``hg`` is also supported.
-
-    ``commit``: string [optional]
-      The commit ID (no branch names, no symbolic refs, no tags) that should
-      be used. If ``url`` was specified but no ``commit``, ``branch`` or
-      ``tag``, the revision you get depends on the defaults of the version
-      control system used.
-
-    ``branch``: string [optional]
-      The upstream branch that should be tracked. If ``commit`` was
-      specified, kas checks that the branch contains the commit. If no
-      ``commit`` was specified, the head of the upstream branch is checked out.
-
-    ``tag``: string [optional]
-      The tag that should be checked out. If a ``commit`` was specified, kas
-      checks that the tag points to this commit. This must not be combined
-      with ``branch``.
-
-    ``path``: string [optional]
-      The path where the repository is stored.
-      If the ``url`` and ``path`` is missing, the repository where the
-      current configuration file is located is defined.
-      If the ``url`` is missing and the path defined, this entry references
-      the directory the path points to.
-      If the ``url`` as well as the ``path`` is defined, the path is used to
-      overwrite the checkout directory, that defaults to ``kas_work_dir``
-      + ``repo.name``.
-      In case of a relative path name ``kas_work_dir`` is prepended.
-
-    ``layers``: dict [optional]
-      Contains the layers from this repository that should be added to the
-      ``bblayers.conf``. If this is missing or ``None`` or an empty
-      dictionary, the path to the repo itself is added as a layer.
-      Additionally, ``.`` is a valid value if the repo itself should be added
-      as a layer. This allows combinations:
-
-      .. code-block:: yaml
-
-         repos:
-           meta-foo:
-             url: https://github.com/bar/meta-foo.git
-             path: layers/meta-foo
-             branch: master
-             layers:
-               .:
-               contrib:
-
-      This adds both ``layers/meta-foo`` and ``layers/meta-foo/contrib`` from
-      the ``meta-foo`` repository to ``bblayers.conf``.
-
-      ``<layer-path>``: enum [optional]
-        Adds the layer with ``<layer-path>`` that is relative to the
-        repository root directory, to the ``bblayers.conf`` if the value of
-        this entry is not in this list: ``['disabled', 'excluded', 'n', 'no',
-        '0', 'false']``. This way it is possible to overwrite the inclusion
-        of a layer in latter loaded configuration files.
-
-    ``patches``: dict [optional]
-      Contains the patches that should be applied to this repo before it is
-      used.
-
-      ``<patches-id>``: dict [optional]
-        One entry in patches with its specific and unique id. All available
-        patch entries are applied in the order of their sorted
-        ``<patches-id>``.
-
-        ``repo``: string [required]
-          The identifier of the repo where the path of this entry is relative
-          to.
-
-        ``path``: string [required]
-          The path to one patch file or a quilt formatted patchset directory.
-
-``overrides``: dict [optional]
-  This object provides a mechanism to override kas configuration items
-  without defining them. By that, only items that already exist are
-  overridden. Note, that all entries below this key are reserved for
-  auto-generation using kas plugins. Do not manually add entries.
-
-  ``repos``: dict [optional]
-    Mapps to the top-level ``repos`` entry.
-
-    ``<repo-id>``: dict [optional]
-      Mapps to the ``<repo-id>`` entry.
-
-    ``commit``: string [optional]
-      Pinned commit ID which overrides the ``commit`` of the corresponding
-      repo.
-
-``bblayers_conf_header``: dict [optional]
-  This contains strings that should be added to the ``bblayers.conf`` before
-  any layers are included.
-
-  ``<bblayers-conf-id>``: string [optional]
-    A string that is added to the ``bblayers.conf``. The entry id
-    (``<bblayers-conf-id>``) should be unique if lines should be added and
-    can be the same from another included file, if this entry should be
-    overwritten. The lines are added to ``bblayers.conf`` in alphabetic order
-    of ``<bblayers-conf-id>`` to ensure deterministic generation of config
-    files.
-
-``local_conf_header``: dict [optional]
-  This contains strings that should be added to the ``local.conf``.
-
-  ``<local-conf-id>``: string [optional]
-    A string that is added to the ``local.conf``. It operates in the same way
-    as the ``bblayers_conf_header`` entry.
-
-``menu_configuration``: dict [optional]
-  This contains user choices for a Kconfig menu of a project. Each variable
-  corresponds to a Kconfig configuration variable and can be of the types
-  string, boolean or integer. The content of this key is typically
-  maintained by the ``kas menu`` plugin in a ``.config.yaml`` file.
-
-``_source_dir``: string [optional]
-  This entry is auto-generated by the menu plugin and provides the path to
-  the top repo at time of invoking the plugin. It must not be set
-  manually and might only be defined in the top-level ``.config.yaml`` file.
-
-``_source_dir_host``: string [optional]
-  This entry is auto-generated by the menu plugin when invoking kas via
-  the ``kas-container`` script. It provides the absolute path to the top repo
-  outside of the container (on the host). This value is only evaluated by the
-  ``kas-container`` script. It must not be set manually and might only be
-  defined in the top-level ``.config.yaml`` file.
diff -pruN 4.3.2-1/docs/userguide/project-configuration.rst 4.4-1/docs/userguide/project-configuration.rst
--- 4.3.2-1/docs/userguide/project-configuration.rst	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/docs/userguide/project-configuration.rst	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,500 @@
+.. _project-configuration-label:
+
+Project Configuration
+=====================
+
+Currently, JSON and YAML are supported as the base file formats. Since YAML is
+arguably easier to read, this documentation focuses on the YAML format.
+
+.. code-block:: yaml
+
+    # Every file needs to contain a header, that provides kas with information
+    # about the context of this file.
+    header:
+      # The `version` entry in the header describes for which configuration
+      # format version this file was created for. It is used by kas to figure
+      # out if it is compatible with this file. The version is an integer that
+      # is increased on every format change.
+      version: x
+    # The machine as it is written into the `local.conf` of bitbake.
+    machine: qemux86-64
+    # The distro name as it is written into the `local.conf` of bitbake.
+    distro: poky
+    repos:
+      # This entry includes the repository where the config file is located
+      # to the bblayers.conf:
+      meta-custom:
+      # Here we include a list of layers from the poky repository to the
+      # bblayers.conf:
+      poky:
+        url: "https://git.yoctoproject.org/git/poky"
+        commit: 89e6c98d92887913cadf06b2adb97f26cde4849b
+        layers:
+          meta:
+          meta-poky:
+          meta-yocto-bsp:
+
+A minimal input file consists out of the ``header``, ``machine``, ``distro``,
+and ``repos``.
+
+Additionally, you can add ``bblayers_conf_header`` and ``local_conf_header``
+which are strings that are added to the head of the respective files
+(``bblayers.conf`` or ``local.conf``):
+
+.. code-block:: yaml
+
+    bblayers_conf_header:
+      meta-custom: |
+        POKY_BBLAYERS_CONF_VERSION = "2"
+        BBPATH = "${TOPDIR}"
+        BBFILES ?= ""
+    local_conf_header:
+      meta-custom: |
+        PATCHRESOLVE = "noop"
+        CONF_VERSION = "1"
+        IMAGE_FSTYPES = "tar"
+
+``meta-custom`` in these examples should be a unique name for this
+configuration entries.
+
+We recommend that this unique name is the **same** as the name of the
+containing repository/layer to ease cross-project referencing.
+
+In given examples we assume that your configuration file is part of a
+``meta-custom`` repository/layer. This way it is possible to overwrite or
+append entries in files that include this configuration by naming an entry
+the same (overwriting) or using an unused name (appending).
+
+Including in-tree configuration files
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+It's currently possible to include kas configuration files from the same
+repository/layer like this:
+
+.. code-block:: yaml
+
+    header:
+      version: x
+      includes:
+        - base.yml
+        - bsp.yml
+        - product.yml
+
+The paths to the files in the include list are either absolute, if they start
+with a `/`, or relative.
+
+If the path is relative and the configuration file is inside a repository,
+then path is relative to the repositories base directory. If the
+configuration file is not in a repository, then the path is relative to the
+parent directory of the file.
+
+Including configuration files from other repos
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+It's also possible to include configuration files from other repos like this:
+
+.. code-block:: yaml
+
+    header:
+      version: x
+      includes:
+        - repo: poky
+          file: kas-poky.yml
+        - repo: meta-bsp-collection
+          file: hw1/kas-hw-bsp1.yml
+        - repo: meta-custom
+          file: products/product.yml
+    repos:
+      meta-custom:
+      meta-bsp-collection:
+        url: "https://www.example.com/git/meta-bsp-collection"
+        commit: 3f786850e387550fdab836ed7e6dc881de23001b
+        layers:
+          # Additional to the layers that are added from this repository
+          # in the hw1/kas-hw-bsp1.yml, we add here an additional bsp
+          # meta layer:
+          meta-custom-bsp:
+      poky:
+        url: "https://git.yoctoproject.org/git/poky"
+        commit: 89e6c98d92887913cadf06b2adb97f26cde4849b
+        layers:
+          # If `kas-poky.yml` adds the `meta-yocto-bsp` layer and we
+          # do not want it in our bblayers for this project, we can
+          # overwrite it by setting:
+          meta-yocto-bsp: excluded
+
+The files are addressed relative to the git repository path.
+
+The include mechanism collects and merges the content from top to bottom and
+depth first. That means that settings in one include file are overwritten
+by settings in a latter include file and entries from the last include file can
+be overwritten by the current file. While merging all the dictionaries are
+merged recursively while preserving the order in which the entries are added to
+the dictionary. This means that ``local_conf_header`` entries are added to the
+``local.conf`` file in the same order in which they are defined in the
+different include files. Note that the order of the configuration file entries
+is not preserved within one include file, because the parser creates normal
+unordered dictionaries.
+
+Including configuration files via the command line
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+When specifying the kas configuration file on the command line, additional
+configurations can be included ad-hoc:
+
+    $ kas build kas-base.yml:debug-image.yml:board.yml
+
+This is equivalent to static inclusion from some kas-combined.yml like this:
+
+.. code-block:: yaml
+
+    header:
+      version: x
+      includes:
+        - kas-base.yml
+        - debug.image.yml
+        - board.yml
+
+Command line inclusion allows to create configurations on-demand, without the
+need to write a kas configuration file for each possible combination.
+
+Note that all configuration files combined via the command line either have to
+come from the same repository or have to live outside of any versioning control.
+kas will refuse any other combination in order to avoid complications and
+configuration flaws that can easily emerge from them.
+
+Working with lockfiles
+~~~~~~~~~~~~~~~~~~~~~~
+
+kas supports the use of lockfiles to pinpoint repositories to exact commit ID
+(e.g. SHA-1 refs for git). A lockfile hereby only overrides the commit ID
+defined in a kas file. When performing the checkout operation (or any other
+operation that performs a checkout), kas checks if a file named
+``<filename>.lock.<ext>`` is found next to the currently processed kas file.
+If this is found, kas loads this file right after processing the current one.
+Note, that this applies to both files on the kas cmdline, as well as included
+files.
+
+The following example shows this mechanism for a file ``kas/kas-isar.yml``
+and its corresponding lockfile ``kas/kas-isar.lock.yml``.
+
+``kas/kas-isar.yml``:
+
+.. code-block:: yaml
+
+  # [...]
+  repos:
+    isar:
+      url: https://github.com/ilbers/isar.git
+      branch: next
+
+``kas/kas-isar.lock.yml``:
+
+.. code-block:: yaml
+
+  header:
+    version: 14
+  overrides:
+    repos:
+      isar:
+        commit: 0336610df8bb0adce76ef8c5a921c758efed9f45
+
+The ``dump`` plugin provides helpers to simplify the creation and update
+of lockfiles. For details, see the plugins documentation: :mod:`kas.plugins.dump`.
+
+Configuration reference
+~~~~~~~~~~~~~~~~~~~~~~~
+
+``header``: dict [required]
+  The header of every kas configuration file. It contains information about
+  the context of the file.
+
+  ``version``: integer [required]
+    Lets kas check if it is compatible with this file. See the
+    :doc:`configuration format changelog <../format-changelog>` for the
+    format history and the latest available version.
+
+  ``includes``: list [optional]
+    A list of configuration files this current file is based on. They are
+    merged in order they are stated. So a latter one could overwrite
+    settings from previous files. The current file can overwrite settings
+    from every included file. An item in this list can have one of two types:
+
+    item: string
+      The path to a kas configuration file, relative to the repository root
+      of the current file.
+
+    item: dict
+      If files from other repositories should be included, choose this
+      representation.
+
+      ``repo``: string [required]
+        The id of the repository where the file is located. The repo
+        needs to be defined in the ``repos`` dictionary as ``<repo-id>``.
+
+      ``file``: string [required]
+        The path to the file, relative to the root of the specified
+        repository.
+
+``build_system``: string [optional]
+  Defines the bitbake-based build system. Known build systems are
+  ``openembedded`` (or ``oe``) and ``isar``. If set, this restricts the
+  search of kas for the init script in the configured repositories to
+  ``oe-init-build-env`` or ``isar-init-build-env``, respectively. If
+  ``kas-container`` finds this property in the top-level kas configuration
+  file (includes are not evaluated), it will automatically select the
+  required container image and invocation mode.
+
+``defaults``: dict [optional]
+  This key can be used to set default values for various properties.
+  This may help you to avoid repeating the same property assignment in
+  multiple places if, for example, you wish to use the same branch for
+  all repositories.
+
+  ``repos``: dict [optional]
+    This key can contain default values for some repository properties.
+    If a default value is set for a repository property it may still be
+    overridden by setting the same property to a different value in a given
+    repository.
+
+    ``branch``: string [optional]
+      Sets the default ``branch`` property applied to all repositories that
+      do not override this.
+
+    ``tag``: string [optional]
+      Sets the default ``tag`` property applied to all repositories that
+      do not override this.
+
+    ``patches``: dict [optional]
+      This key can contain default values for some repository patch
+      properties. If a default value is set for a patch property it may
+      still be overridden by setting the same property to a different value
+      in a given patch.
+
+      ``repo``: string [optional]
+        Sets the default ``repo`` property applied to all repository
+        patches that do not override this.
+
+``machine``: string [optional]
+  Contains the value of the ``MACHINE`` variable that is written into the
+  ``local.conf``. Can be overwritten by the ``KAS_MACHINE`` environment
+  variable and defaults to ``qemux86-64``.
+
+``distro``: string [optional]
+  Contains the value of the ``DISTRO`` variable that is written into the
+  ``local.conf``. Can be overwritten by the ``KAS_DISTRO`` environment
+  variable and defaults to ``poky``.
+
+``target``: string [optional] or list [optional]
+  Contains the target or a list of targets to build by bitbake. Can be
+  overwritten by the ``KAS_TARGET`` environment variable and defaults to
+  ``core-image-minimal``. Space is used as a delimiter if multiple targets
+  should be specified via the environment variable.
+  For targets prefixed with ``multiconfig:`` or ``mc:``, corresponding
+  entries are added to the ``BBMULTICONFIG`` in ``local.conf``.
+
+``env``: dict [optional]
+  Contains environment variable names with either default values or ``null``.
+  These variables are made available to bitbake via
+  ``BB_ENV_PASSTHROUGH_ADDITIONS`` (``BB_ENV_EXTRAWHITE`` in older Bitbake
+  versions) and can be overwritten by the variables of the environment in
+  which kas is started.
+  Either a string or nothing (``null``) can be assigned as value.
+  The former one serves as a default value whereas the latter one will lead
+  to add the variable only to ``BB_ENV_PASSTHROUGH_ADDITIONS`` and not to
+  the environment where kas is started. Please note, that ``null`` needs to
+  be assigned as the nulltype (e.g. ``MYVAR: null``), not as 'null'.
+
+``task``: string [optional]
+  Contains the task to build by bitbake. Can be overwritten by the
+  ``KAS_TASK`` environment variable and defaults to ``build``.
+
+``repos``: dict [optional]
+  Contains the definitions of all available repos and layers.
+
+  ``<repo-id>``: dict [optional]
+    Contains the definition of a repository and the layers, that should be
+    part of the build. If the value is ``None``, the repository, where the
+    current configuration file is located is defined as ``<repo-id>`` and
+    added as a layer to the build. It is recommended that the ``<repo-id>``
+    is related to the containing repository/layer to ease cross-project
+    referencing.
+
+    ``name``: string [optional]
+      Defines under which name the repository is stored. If its missing
+      the ``<repo-id>`` will be used.
+
+    ``url``: string [optional]
+      The url of the repository. If this is missing, no version control
+      operations are performed.
+
+    ``type``: string [optional]
+      The type of version control repository. The default value is ``git``
+      and ``hg`` is also supported.
+
+    ``commit``: string [optional]
+      The commit ID (no branch names, no symbolic refs, no tags) that should
+      be used. If ``url`` was specified but no ``commit``, ``branch`` or
+      ``tag``, the revision you get depends on the defaults of the version
+      control system used.
+
+    ``branch``: string [optional]
+      The upstream branch that should be tracked. If ``commit`` was
+      specified, kas checks that the branch contains the commit. If no
+      ``commit`` was specified, the head of the upstream branch is checked out.
+
+    ``tag``: string [optional]
+      The tag that should be checked out. If a ``commit`` was specified, kas
+      checks that the tag points to this commit. This must not be combined
+      with ``branch``.
+
+    ``path``: string [optional]
+      The path where the repository is stored.
+      If the ``url`` and ``path`` is missing, the repository where the
+      current configuration file is located is defined.
+      If the ``url`` is missing and the path defined, this entry references
+      the directory the path points to.
+      If the ``url`` as well as the ``path`` is defined, the path is used to
+      overwrite the checkout directory, that defaults to ``kas_work_dir``
+      + ``repo.name``.
+      In case of a relative path name ``kas_work_dir`` is prepended.
+
+    ``layers``: dict [optional]
+      Contains the layers from this repository that should be added to the
+      ``bblayers.conf``. If this is missing or ``None`` or an empty
+      dictionary, the path to the repo itself is added as a layer.
+      Additionally, ``.`` is a valid value if the repo itself should be added
+      as a layer. This allows combinations:
+
+      .. code-block:: yaml
+
+         repos:
+           meta-foo:
+             url: https://github.com/bar/meta-foo.git
+             path: layers/meta-foo
+             branch: master
+             layers:
+               .:
+               contrib:
+
+      This adds both ``layers/meta-foo`` and ``layers/meta-foo/contrib`` from
+      the ``meta-foo`` repository to ``bblayers.conf``.
+
+      ``<layer-path>``: enum [optional]
+        Adds the layer with ``<layer-path>`` that is relative to the
+        repository root directory, to the ``bblayers.conf`` if the value of
+        this entry is not in this list: ``['disabled', 'excluded', 'n', 'no',
+        '0', 'false']``. This way it is possible to overwrite the inclusion
+        of a layer in latter loaded configuration files.
+
+    ``patches``: dict [optional]
+      Contains the patches that should be applied to this repo before it is
+      used.
+
+      ``<patches-id>``: dict [optional]
+        One entry in patches with its specific and unique id. All available
+        patch entries are applied in the order of their sorted
+        ``<patches-id>``.
+
+        ``repo``: string [required]
+          The identifier of the repo where the path of this entry is relative
+          to.
+
+        ``path``: string [required]
+          The path to one patch file or a quilt formatted patchset directory.
+
+``overrides``: dict [optional]
+  This object provides a mechanism to override kas configuration items
+  without defining them. By that, only items that already exist are
+  overridden. Note, that all entries below this key are reserved for
+  auto-generation using kas plugins. Do not manually add entries.
+
+  ``repos``: dict [optional]
+    Mapps to the top-level ``repos`` entry.
+
+    ``<repo-id>``: dict [optional]
+      Mapps to the ``<repo-id>`` entry.
+
+    ``commit``: string [optional]
+      Pinned commit ID which overrides the ``commit`` of the corresponding
+      repo.
+
+``bblayers_conf_header``: dict [optional]
+  This contains strings that should be added to the ``bblayers.conf`` before
+  any layers are included.
+
+  ``<bblayers-conf-id>``: string [optional]
+    A string that is added to the ``bblayers.conf``. The entry id
+    (``<bblayers-conf-id>``) should be unique if lines should be added and
+    can be the same from another included file, if this entry should be
+    overwritten. The lines are added to ``bblayers.conf`` in alphabetic order
+    of ``<bblayers-conf-id>`` to ensure deterministic generation of config
+    files.
+
+``local_conf_header``: dict [optional]
+  This contains strings that should be added to the ``local.conf``.
+
+  ``<local-conf-id>``: string [optional]
+    A string that is added to the ``local.conf``. It operates in the same way
+    as the ``bblayers_conf_header`` entry.
+
+``menu_configuration``: dict [optional]
+  This contains user choices for a Kconfig menu of a project. Each variable
+  corresponds to a Kconfig configuration variable and can be of the types
+  string, boolean or integer. The content of this key is typically
+  maintained by the ``kas menu`` plugin in a ``.config.yaml`` file.
+
+``artifacts``: dict [optional]
+  This entry describes artifacts which are expected to be present after
+  executing the build. Each key-value pair describes an identifier and a
+  path relative to the kas build dir, whereby the path can contain wildcards
+  like '*'. Unix-style globbing is applied to all paths. In case no artifact
+  is found, the build is considered successful, if not stated otherwise by
+  the used plugin and mode of operation.
+
+  .. note:: There are no further semantics attached to the identifiers (yet).
+            Both the author and the consumer of the artifacts node need to
+            agree on the semantics.
+
+  Example:
+
+  .. code-block:: yaml
+
+      artifacts:
+        disk-image: path/to/image.*.img
+        firmware: path/to/firmware.bin
+        swu: path/to/update.swu
+
+``_source_dir``: string [optional]
+  This entry is auto-generated by the menu plugin and provides the path to
+  the top repo at time of invoking the plugin. It must not be set
+  manually and might only be defined in the top-level ``.config.yaml`` file.
+
+``_source_dir_host``: string [optional]
+  This entry is auto-generated by the menu plugin when invoking kas via
+  the ``kas-container`` script. It provides the absolute path to the top repo
+  outside of the container (on the host). This value is only evaluated by the
+  ``kas-container`` script. It must not be set manually and might only be
+  defined in the top-level ``.config.yaml`` file.
+
+.. _example-configurations-label:
+
+Example project configurations
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+The following snippets show minimal but working project configurations for
+both OpenEmbedded and ISAR based distributions.
+
+OpenEmbedded
+------------
+
+.. literalinclude:: ../../examples/openembedded.yml
+  :language: YAML
+  :lines: 25-
+
+ISAR
+----
+
+.. literalinclude:: ../../examples/isar.yml
+  :language: YAML
+  :lines: 25-
diff -pruN 4.3.2-1/docs/userguide.rst 4.4-1/docs/userguide.rst
--- 4.3.2-1/docs/userguide.rst	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/docs/userguide.rst	2024-05-31 16:23:43.000000000 +0000
@@ -1,159 +1,11 @@
 User Guide
 ==========
 
-.. only:: html
+.. toctree::
+  :maxdepth: 2
 
-  Dependencies & installation
-  ---------------------------
-
-  This project depends on
-
-  - Python 3
-  - distro Python 3 package
-  - jsonschema Python 3 package
-  - PyYAML Python 3 package
-  - GitPython Python 3 package
-  - kconfiglib Python 3 package (optional, for menu plugin)
-  - NEWT Python 3 distro package (optional, for menu plugin)
-
-  To install kas into your python site-package repository, run::
-
-      $ sudo pip3 install .
-
-
-Usage
------
-
-.. only:: html
-
-  There are (at least) four options for using kas:
-
-  - Install it locally via pip to get the ``kas`` command.
-  - Use the container image locally. In this case, download the ``kas-container``
-    script from the kas repository and use it in place of the ``kas`` command.
-    The script version corresponds to the kas tool and the kas image version.
-  - Use the container image in CI. Specify
-    ``ghcr.io/siemens/kas/kas[-isar][:<x.y>]`` in your CI script that requests
-    a container image as runtime environment. See
-    https://github.com/orgs/siemens/packages/container/kas%2Fkas/versions and
-    https://github.com/orgs/siemens/packages/container/kas%2Fkas-isar/versions for
-    all available images.
-  - Use the **run-kas** wrapper from this directory. In this case,
-    replace ``kas`` in the examples below with ``path/to/run-kas``.
-
-Start build::
-
-    $ kas build /path/to/kas-project.yml
-
-Alternatively, experienced bitbake users can invoke usual **bitbake** steps
-manually, e.g.::
-
-    $ kas shell /path/to/kas-project.yml -c 'bitbake dosfsutils-native'
-
-kas will place downloads and build artifacts under the current directory when
-being invoked. You can specify a different location via the environment
-variable `KAS_WORK_DIR`.
-
-
-Use Cases
----------
-
-1.  Initial build/setup::
-
-    $ mkdir $PROJECT_DIR
-    $ cd $PROJECT_DIR
-    $ git clone $PROJECT_URL meta-project
-    $ kas build meta-project/kas-project.yml
-
-2.  Update/rebuild::
-
-    $ cd $PROJECT_DIR/meta-project
-    $ git pull
-    $ kas build kas-project.yml
-
-3.  Interactive configuration::
-
-    $ cd $PROJECT_DIR/meta-project
-    $ kas menu
-    $ kas build  # optional, if not triggered via kas menu
-
-
-Plugins
--------
-
-kas sub-commands are implemented by a series of plugins. Each plugin
-typically provides a single command.
-
-.. only:: html
-
-  ``build`` plugin
-  ~~~~~~~~~~~~~~~~
-
-  .. include:: userguide/plugins/build.inc
-
-  ``checkout`` plugin
-  ~~~~~~~~~~~~~~~~~~~
-
-  .. include:: userguide/plugins/checkout.inc
-
-  ``dump`` plugin
-  ~~~~~~~~~~~~~~~
-
-  .. include:: userguide/plugins/dump.inc
-
-  ``for-all-repos`` plugin
-  ~~~~~~~~~~~~~~~~~~~~~~~~
-
-  .. include:: userguide/plugins/for-all-repos.inc
-
-  ``menu`` plugin
-  ~~~~~~~~~~~~~~~
-
-  .. include:: userguide/plugins/menu.inc
-
-  ``shell`` plugin
-  ~~~~~~~~~~~~~~~~
-
-  .. include:: userguide/plugins/shell.inc
-
-.. only:: man
-
-  :manpage:`kas-build(1)`
-    build the project
-  :manpage:`kas-checkout(1)`
-    checkout all repos without building
-  :manpage:`kas-dump(1)`
-    dump the flattened configuration or lockfiles
-  :manpage:`kas-for-all-repos(1)`
-    run a command in each repository
-  :manpage:`kas-menu(1)`
-    interactive menu to configure the build
-  :manpage:`kas-shell(1)`
-    start a shell in the build environment
-
-Project Configuration
----------------------
-
-.. only:: html
-
-  .. include:: userguide/project-configuration.inc
-
-.. only:: man
-
-  The project configuration file describes the build environment and the layers
-  to be used. It is the main input to kas.
-  For details, see :manpage:`kas-project-config(1)`
-
-.. _checkout-creds-label:
-
-Credential Handling
--------------------
-
-.. only:: html
-
-  .. include:: userguide/credentials.inc
-
-.. only:: man
-
-  kas provides various mechanisms to inject credentials into the build.
-  For details, see :manpage:`kas-credentials(1)`.
+  userguide/getting-started
+  userguide/plugins
+  userguide/project-configuration
+  userguide/build-attestation
+  userguide/credentials
diff -pruN 4.3.2-1/examples/isar.yml 4.4-1/examples/isar.yml
--- 4.3.2-1/examples/isar.yml	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/examples/isar.yml	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,41 @@
+#
+# kas - setup tool for bitbake based projects
+#
+# Copyright (c) Siemens AG, 2022-2024
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be
+# included in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
+#
+
+header:
+  version: 14
+
+build_system: isar
+
+machine: qemuamd64
+distro: debian-bullseye
+
+target: mc:qemuamd64-bullseye:cowsay
+
+repos:
+  isar:
+    url: https://github.com/ilbers/isar.git
+    commit: 9dc362cd7115074c0f60843e687ed89db5c9339a
+    layers:
+      meta:
+      meta-isar:
diff -pruN 4.3.2-1/examples/openembedded.yml 4.4-1/examples/openembedded.yml
--- 4.3.2-1/examples/openembedded.yml	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/examples/openembedded.yml	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,39 @@
+#
+# kas - setup tool for bitbake based projects
+#
+# Copyright (c) Siemens AG, 2022-2024
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be
+# included in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
+#
+
+header:
+  version: 14
+
+machine: qemux86-64
+distro: poky
+
+target: zlib-native
+
+repos:
+  poky:
+    url: https://git.yoctoproject.org/poky.git
+    commit: fb91a49387cfb0c8d48303bb3354325ba2a05587
+    layers:
+      meta:
+      meta-poky:
diff -pruN 4.3.2-1/image-tests/kas/kas.yml 4.4-1/image-tests/kas/kas.yml
--- 4.3.2-1/image-tests/kas/kas.yml	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/image-tests/kas/kas.yml	2024-05-31 16:23:43.000000000 +0000
@@ -1,7 +1,7 @@
 #
 # kas - setup tool for bitbake based projects
 #
-# Copyright (c) Siemens AG, 2022-2023
+# Copyright (c) Siemens AG, 2022-2024
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -25,12 +25,15 @@
 header:
   version: 14
 
+machine: qemux86-64
+distro: poky
+
 target: zlib-native
 
 repos:
   poky:
     url: https://git.yoctoproject.org/poky.git
-    commit: 387ab5f18b17c3af3e9e30dc58584641a70f359f
+    commit: fb91a49387cfb0c8d48303bb3354325ba2a05587
     layers:
       meta:
       meta-poky:
diff -pruN 4.3.2-1/image-tests/kas-isar/kas.yml 4.4-1/image-tests/kas-isar/kas.yml
--- 4.3.2-1/image-tests/kas-isar/kas.yml	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/image-tests/kas-isar/kas.yml	2024-05-31 16:23:43.000000000 +0000
@@ -1,7 +1,7 @@
 #
 # kas - setup tool for bitbake based projects
 #
-# Copyright (c) Siemens AG, 2022-2023
+# Copyright (c) Siemens AG, 2022-2024
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -35,7 +35,7 @@ target: mc:qemuamd64-bullseye:cowsay
 repos:
   isar:
     url: https://github.com/ilbers/isar.git
-    commit: 8dfa97cf6e25a7adec2e02c50c5ef6329f31931b
+    commit: 9dc362cd7115074c0f60843e687ed89db5c9339a
     layers:
       meta:
       meta-isar:
diff -pruN 4.3.2-1/kas/__version__.py 4.4-1/kas/__version__.py
--- 4.3.2-1/kas/__version__.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/kas/__version__.py	2024-05-31 16:23:43.000000000 +0000
@@ -25,8 +25,8 @@
 __license__ = 'MIT'
 __copyright__ = 'Copyright (c) Siemens AG, 2017-2020'
 
-__version__ = '4.3.2'
+__version__ = '4.4'
 
 # Please update docs/format-changelog.rst when changing the file version.
-__file_version__ = 16
+__file_version__ = 17
 __compatible_file_version__ = 1
diff -pruN 4.3.2-1/kas/attestation.py 4.4-1/kas/attestation.py
--- 4.3.2-1/kas/attestation.py	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/kas/attestation.py	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,244 @@
+# kas - setup tool for bitbake based projects
+#
+# Copyright (c) Siemens AG, 2024
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be
+# included in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
+"""
+    This module provides infrastructure to generate provenance attestation
+    of the build process.
+"""
+
+import os
+import distro
+import logging
+import hashlib
+import base64
+import sys
+from enum import Enum
+from urllib.parse import urlparse
+from pathlib import Path
+from datetime import datetime, timezone
+from kas import __version__ as KASVERSION
+
+
+SLSA_PROVENANCE_TYPE = 'https://slsa.dev/provenance/v1'
+KAS_BUILDER_ID = 'https://github.com/siemens/kas'
+KAS_BUILD_TYPE = 'https://kas.readthedocs.io/en' \
+                 f'/{KASVERSION}/userguide.html#project-configuration'
+INTOTO_STATEMENT_TYPE = 'https://in-toto.io/Statement/v1'
+
+
+def date_to_rfc3339(dt):
+    return dt.astimezone(timezone.utc) \
+             .strftime('%Y-%m-%dT%H:%M:%S.%fZ')
+
+
+def file_digest_slow(f, algorithm):
+    """
+        Implementation of hashlib.file_digest for python < 3.11
+    """
+    hash = hashlib.new(algorithm)
+    while True:
+        data = f.read()
+        if not data:
+            break
+        hash.update(data)
+    return hash
+
+
+class Provenance:
+    """
+        Create a build attestation predicate in SLSA provenance format
+        for a kas build.
+    """
+    class Mode(Enum):
+        MIN = 1
+        MAX = 10
+
+    def __init__(self, ctx, t_started, t_finished, mode=Mode.MIN):
+        self._ctx = ctx
+        self._t_started = t_started
+        self._t_finished = t_finished
+        self._mode = mode
+
+    @staticmethod
+    def _url_with_protocol(url):
+        if url.startswith('git@'):
+            return f'ssh://{url}'
+        if url.startswith('ssh://'):
+            return f'{url}'
+        if url.startswith('http://') or url.startswith('https://'):
+            return f'{url}'
+
+    @staticmethod
+    def _strip_credentials(url):
+        """
+            Returns the url with all credentials removed (best effort)
+        """
+        if not url.startswith('http://') and not url.startswith('https://'):
+            return url
+        parsed = urlparse(url)
+        if parsed.username and parsed.password:
+            url = url.replace(f'{parsed.username}:{parsed.password}@', '')
+        return url
+
+    @staticmethod
+    def _get_filetype(f: Path):
+        if f.suffix == '.json':
+            return 'json'
+        return 'yaml'
+
+    def _make_relative_path(self, path: Path):
+        top_repo_path = Path(self._ctx.config.handler.get_top_repo_path())
+        workdir = Path(self._ctx.kas_work_dir)
+
+        # is_relative_to implementation for python < 3.9
+        try:
+            return path.relative_to(workdir)
+        except ValueError:
+            return path.relative_to(top_repo_path)
+
+    def type_(self):
+        return SLSA_PROVENANCE_TYPE
+
+    def as_dict(self):
+        res_deps = []
+        for r in self._ctx.config.get_repos():
+            if r.operations_disabled:
+                if not r.url or not r.revision:
+                    continue
+            digest = {'sha1': r.revision}
+            annotations = {
+                'dirty': r.dirty,
+                'layers': [str(Path(layer).relative_to(r.path))
+                           for layer in r.layers]
+            }
+            cleanurl = self._strip_credentials(r.url)
+            dep = {
+                'name': r.name,
+                'uri': f'{r.get_type()}+{self._url_with_protocol(cleanurl)}',
+                'digest': digest,
+                'annotations': annotations
+            }
+            res_deps.append(dep)
+
+        # (abspath, relpath)
+        config_files = [(Path(c), self._make_relative_path(Path(c)))
+                        for c in self._ctx.config.filenames]
+        for ca, cr in config_files:
+            with open(ca, 'rb') as f:
+                content = f.read()
+            rd = {
+                'name': str(cr),
+                'content': base64.b64encode(content).decode('utf-8'),
+                'mediaType': f'application/vnd.kas+{self._get_filetype(ca)}'
+            }
+            res_deps.append(rd)
+
+        bd = {
+            'buildType': KAS_BUILD_TYPE,
+            'externalParameters': {
+                'command': self._ctx.args.cmd,
+                'config': [str(c) for _, c in config_files],
+                'target': self._ctx.args.target,
+                'task': self._ctx.args.task,
+                'extra_bitbake_args': self._ctx.args.extra_bitbake_args,
+            },
+            'internalParameters': {},
+            'resolvedDependencies': res_deps
+        }
+        if self._mode == self.Mode.MAX:
+            bd['internalParameters']['env'] = \
+                self._ctx.config.get_environment()
+        b_versions = {
+            'kas': KASVERSION,
+            'distro.name': distro.id(),
+            'distro.version': distro.version()
+        }
+
+        rd = {
+            'builder':
+            {
+                'id': KAS_BUILDER_ID,
+                'version': b_versions
+            },
+            'metadata': {
+                'invocationId': os.environ.get('CI_JOB_URL', ''),
+                'startedOn': date_to_rfc3339(self._t_started),
+                'finishedOn': date_to_rfc3339(self._t_finished),
+            },
+            'byproducts': []
+        }
+        p = {
+            'buildDefinition': bd,
+            'runDetails': rd
+        }
+        return p
+
+
+class Statement:
+    """
+        Create a statement in in-toto format for a kas build.
+    """
+    def __init__(self, predicate, ctx, t_started, t_finished):
+        self._predicate = predicate
+        self._ctx = ctx
+        self._t_started = t_started
+        self._t_finished = t_finished
+
+    def _check_artifact_timestamp(self, name, path):
+        """
+            Warn if artifact timestamp is not within the build range.
+        """
+        logging.debug(f'Found artifact {name}:{path} in build dir')
+        fullpath = Path(self._ctx.build_dir) / path
+        mtime = datetime.fromtimestamp(fullpath.stat().st_mtime)
+        if mtime < self._t_started or mtime > self._t_finished:
+            logging.warning(
+                f'Artifact {name}:{path.name} mtime {mtime.strftime("%c")}'
+                f' not in build range '
+                f'[{self._t_started.strftime("%c")} - '
+                f'{self._t_finished.strftime("%c")}]')
+
+    def as_dict(self):
+        pt = self._predicate.type_()
+        pp = self._predicate.as_dict()
+        subjects = []
+        for n, s in self._ctx.config.get_artifacts(missing_ok=False):
+            self._check_artifact_timestamp(n, s)
+            fullpath = Path(self._ctx.build_dir) / s
+            with open(fullpath, "rb") as f:
+                if sys.version_info < (3, 11):
+                    digest = file_digest_slow(f, "sha256")
+                else:
+                    digest = hashlib.file_digest(f, "sha256")
+            rd = {
+                'name': s.name,
+                'digest': {'sha256': digest.hexdigest()}
+            }
+            subjects.append(rd)
+        if len(subjects) == 0:
+            logging.warning('Attestation does not contain any artifacts.')
+        st = {
+            '_type': INTOTO_STATEMENT_TYPE,
+            'subject': subjects,
+            'predicateType': pt,
+            'predicate': pp
+        }
+        return st
diff -pruN 4.3.2-1/kas/config.py 4.4-1/kas/config.py
--- 4.3.2-1/kas/config.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/kas/config.py	2024-05-31 16:23:43.000000000 +0000
@@ -25,8 +25,10 @@
 
 import os
 import json
+from pathlib import Path
 from .repos import Repo
 from .includehandler import IncludeHandler, IncludeException
+from .kasusererror import ArtifactNotFoundError
 
 __license__ = 'MIT'
 __copyright__ = 'Copyright (c) Siemens AG, 2017-2021'
@@ -41,6 +43,7 @@ class Config:
     def __init__(self, ctx, filename, target=None, task=None):
         self._override_target = target
         self._override_task = task
+        self._build_dir = ctx.build_dir
         self._config = {}
         if not filename:
             filename = os.path.join(ctx.kas_work_dir, CONFIG_YAML_FILE)
@@ -219,3 +222,20 @@ class Config:
             if target.startswith('multiconfig:') or target.startswith('mc:'):
                 multiconfigs.add(target.split(':')[1])
         return ' '.join(multiconfigs)
+
+    def get_artifacts(self, missing_ok=True):
+        """
+            Returns the found artifacts after glob expansion, relative
+            to the build_dir as a list of tuples (name, path).
+            If missing_ok=False, raises an ArtifactNotFoundError if no
+            artifact for a given name is found.
+        """
+        arts = self._config.get('artifacts', {})
+        foundfiles = []
+        for name, art in arts.items():
+            files = list(Path(self._build_dir).glob(art))
+            if not missing_ok and len(files) == 0:
+                raise ArtifactNotFoundError(name, art)
+            foundfiles.extend([(name, f) for f in files])
+        return [(n, f.relative_to(self._build_dir))
+                for n, f in foundfiles]
diff -pruN 4.3.2-1/kas/context.py 4.4-1/kas/context.py
--- 4.3.2-1/kas/context.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/kas/context.py	2024-05-31 16:23:43.000000000 +0000
@@ -25,6 +25,7 @@
 
 import os
 import logging
+from kas.kasusererror import KasUserError
 
 try:
     import distro
@@ -78,6 +79,10 @@ class Context:
         self.__kas_build_dir = os.path.abspath(build_dir)
         ref_dir = os.environ.get('KAS_REPO_REF_DIR', None)
         self.__kas_repo_ref_dir = os.path.abspath(ref_dir) if ref_dir else None
+        clone_depth = os.environ.get('KAS_CLONE_DEPTH', '0')
+        if not clone_depth.isdigit():
+            raise KasUserError('KAS_CLONE_DEPTH must be a number')
+        self.repo_clone_depth = max(int(clone_depth), 0)
         self.setup_initial_environ()
         self.config = None
         self.args = args
diff -pruN 4.3.2-1/kas/kas.py 4.4-1/kas/kas.py
--- 4.3.2-1/kas/kas.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/kas/kas.py	2024-05-31 16:23:43.000000000 +0000
@@ -141,12 +141,32 @@ def kas_get_argparser():
     subparser = parser.add_subparsers(help='sub command help', dest='cmd')
 
     for plugin in plugins.all():
-        plugin_parser = subparser.add_parser(plugin.name, help=plugin.helpmsg)
+        plugin_parser = subparser.add_parser(
+            plugin.name,
+            help=plugin.helpmsg,
+            formatter_class=ArgumentChoicesHelpFormatter)
         plugin.setup_parser(plugin_parser)
 
     return parser
 
 
+class ArgumentChoicesHelpFormatter(argparse.HelpFormatter):
+    """Help message formatter which adds choices to argument help.
+
+    If the default METAVAR is used, this will do nothing, as the default
+    METAVAR shows the available choices already. If the METAVAR is
+    overridden, and %(choice)s is not present in the help string, add
+    them.
+    """
+
+    def _get_help_string(self, action):
+        help = action.help
+        if action.choices and action.metavar is not None:
+            if "%(choices)" not in action.help:
+                help += " Possible choices: %(choices)s."
+        return help
+
+
 def kas(argv):
     """
         The actual main entry point of kas.
diff -pruN 4.3.2-1/kas/kasusererror.py 4.4-1/kas/kasusererror.py
--- 4.3.2-1/kas/kasusererror.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/kas/kasusererror.py	2024-05-31 16:23:43.000000000 +0000
@@ -66,3 +66,19 @@ class ArgsCombinationError(KasUserError)
     """
     def __init__(self, message):
         super().__init__(f'Invalid combination of arguments: {message}')
+
+
+class ArtifactNotFoundError(KasUserError, FileNotFoundError):
+    """
+    A configured artifact is not found (or the glob matches 0 elements).
+    """
+    def __init__(self, name, artifact):
+        super().__init__(f'No artifact found for {name}:"{artifact}"')
+
+
+class MissingModuleError(KasUserError):
+    """
+    An optional module is missing for the requested operation
+    """
+    def __init__(self, module, operation) -> None:
+        super().__init__(f'Module "{module}" is required for: {operation}')
diff -pruN 4.3.2-1/kas/libcmds.py 4.4-1/kas/libcmds.py
--- 4.3.2-1/kas/libcmds.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/kas/libcmds.py	2024-05-31 16:23:43.000000000 +0000
@@ -180,6 +180,35 @@ class SetupHome(Command):
     def __str__(self):
         return 'setup_home'
 
+    @staticmethod
+    def _on_ci():
+        """
+            Detects if we are running on a CI system.
+            Returns the name of the CI system or None.
+        """
+        if os.environ.get('GITHUB_ACTIONS', False) == 'true':
+            return 'GitHub Actions'
+        elif os.environ.get('GITLAB_CI', False) == 'true':
+            return 'GitLab CI'
+        return None
+
+    @staticmethod
+    def _ssh_config_present():
+        """
+            Checks if any file in the .ssh dir exists or
+            any manual ssh config option is set.
+        """
+        ssh_vars = ['SSH_PRIVATE_KEY', 'SSH_PRIVATE_KEY_FILE', 'SSH_AUTH_SOCK']
+        if any(e in os.environ for e in ssh_vars):
+            return True
+
+        ssh_path = os.path.expanduser('~/.ssh')
+        if os.path.isdir(ssh_path):
+            with os.scandir(ssh_path) as it:
+                if any(it):
+                    return True
+        return False
+
     def _setup_netrc(self):
         if os.environ.get('NETRC_FILE', False):
             shutil.copy(os.environ['NETRC_FILE'],
@@ -222,9 +251,8 @@ class SetupHome(Command):
         gitconfig_host = os.environ.get('GITCONFIG_FILE', False)
         gitconfig_kas = self.tmpdirname + '/.gitconfig'
 
-        # when running in the github ci, always try to read the gitconfig
-        if not gitconfig_host and \
-           os.environ.get('GITHUB_ACTIONS', False) == 'true':
+        # on supported CI systems, always try to read the gitconfig
+        if not gitconfig_host and self._on_ci():
             gitconfig_host = os.path.expanduser('~/.gitconfig')
 
         if gitconfig_host and os.path.exists(gitconfig_host):
@@ -243,9 +271,23 @@ class SetupHome(Command):
                 if os.environ.get('GIT_CREDENTIAL_USEHTTPPATH', False):
                     config['credential']['useHttpPath'] = \
                         os.environ.get('GIT_CREDENTIAL_USEHTTPPATH')
+            # in GitLab CI, add ssh -> https rewrites if no config is present
+            ci_server = os.environ.get('CI_SERVER_HOST', False)
+            if self._on_ci() == 'GitLab CI' and ci_server and \
+                    not self._ssh_config_present() and \
+                    not os.path.exists(gitconfig_host):
+                logging.debug('Adding GitLab CI ssh -> https rewrites')
+                section = f'url "https://{ci_server}/"'
+                config.add_value(section, 'insteadOf',
+                                 f'git@{ci_server}:')
+                config.add_value(section, 'insteadOf',
+                                 f'ssh://git@{ci_server}/')
             config.write()
 
     def execute(self, ctx):
+        ci = self._on_ci()
+        if ci:
+            logging.info(f'Running on {ci}')
         def_umask = os.umask(0o077)
         self._setup_netrc()
         self._setup_gitconfig()
diff -pruN 4.3.2-1/kas/libkas.py 4.4-1/kas/libkas.py
--- 4.3.2-1/kas/libkas.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/kas/libkas.py	2024-05-31 16:23:43.000000000 +0000
@@ -23,6 +23,7 @@
     This module contains the core implementation of kas.
 """
 
+import argparse
 import re
 import os
 import sys
@@ -406,13 +407,19 @@ def ssh_no_host_key_check():
 
 
 def setup_parser_common_args(parser):
+    from kas.libcmds import Macro
+
     parser.add_argument('config',
                         help='Config file(s), separated by colon. Using '
                         '.config.yaml in KAS_WORK_DIR if none is specified.',
                         nargs='?')
     parser.add_argument('--skip',
-                        help='Skip build steps',
-                        default=[])
+                        help='Skip build steps. To skip more than one step, '
+                        'use this argument multiple times.',
+                        default=[],
+                        action='append',
+                        metavar='STEP',
+                        choices=[str(c) for c in Macro().setup_commands])
     parser.add_argument('--force-checkout', action='store_true',
                         help='Always checkout the desired commit/branch/tag '
                         'of each repository, discarding any local changes')
@@ -427,6 +434,48 @@ def setup_parser_preserve_env_arg(parser
                         action='store_true')
 
 
+class ExtendConstAction(argparse._AppendConstAction):
+    """Add an 'extend_const' action similar to 'append_const'.
+
+    Based on the existing 'append_const' and 'extend' actions.
+    """
+    def __init__(self, option_strings, dest, const, default=None,
+                 required=False, help=None, metavar=None):
+        super(argparse._AppendConstAction, self).__init__(
+            option_strings=option_strings, dest=dest, nargs=0, const=const,
+            default=default, required=required, help=help, metavar=metavar)
+
+    def __call__(self, parser, namespace, values, option_string=None):
+        items = getattr(namespace, self.dest, None)
+        if items is None:
+            items = []
+
+        if isinstance(items, list):
+            items = items[:]
+        else:
+            import copy
+            items = copy.copy(items)
+
+        items.extend(self.const)
+        setattr(namespace, self.dest, items)
+
+
+def setup_parser_keep_config_unchanged_arg(parser):
+    # Skip the tasks which would change the config of the build
+    # environment
+    steps = [
+        'setup_dir',
+        'finish_setup_repos',
+        'repos_apply_patches',
+        'write_bbconfig',
+    ]
+    parser.add_argument('-k', '--keep-config-unchanged',
+                        help='Skip steps that change the configuration',
+                        action=ExtendConstAction,
+                        dest='skip',
+                        const=steps)
+
+
 def run_handle_preserve_env_arg(ctx, os, args, SetupHome):
     if args.preserve_env:
         # Warn if there's any settings that setup_home would apply
diff -pruN 4.3.2-1/kas/plugins/build.py 4.4-1/kas/plugins/build.py
--- 4.3.2-1/kas/plugins/build.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/kas/plugins/build.py	2024-05-31 16:23:43.000000000 +0000
@@ -1,6 +1,6 @@
 # kas - setup tool for bitbake based projects
 #
-# Copyright (c) Siemens AG, 2017-2018
+# Copyright (c) Siemens AG, 2017-2024
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -26,24 +26,34 @@
     build environment and then invoke bitbake to build the targets selected
     in the chosen config file.
 
-    For example, to build the configuration described in the file
-    ``kas-project.yml`` you could run::
-
-        kas build kas-project.yml
+    When running with ``--provenance <true|mode=...>`` kas will generate an
+    provenance attestation for the build. The attestation will be stored in
+    ``attestation/kas-build.provenance.json`` in the build directory.
+    For details about provenance, see the build attestation chapter.
+
+    .. note::
+        In provenance mode, the command returns with a non-zero exit
+        code in case no artifact is found for at least one entry.
 """
 
 import logging
 import subprocess
 import sys
+import json
+from pathlib import Path
+from datetime import datetime
 from kas.context import create_global_context
 from kas.config import Config
 from kas.libkas import find_program, run_cmd
+from kas.libkas import setup_parser_keep_config_unchanged_arg
 from kas.libcmds import Macro, Command
 from kas.libkas import setup_parser_common_args
 from kas.kasusererror import CommandExecError
+from kas.attestation import Provenance, Statement
+
 
 __license__ = 'MIT'
-__copyright__ = 'Copyright (c) Siemens AG, 2017-2018'
+__copyright__ = 'Copyright (c) Siemens AG, 2017-2024'
 
 
 class Build:
@@ -64,6 +74,7 @@ class Build:
         """
 
         setup_parser_common_args(parser)
+        setup_parser_keep_config_unchanged_arg(parser)
         parser.add_argument('extra_bitbake_args',
                             nargs='*',
                             help='Extra arguments to pass to bitbake '
@@ -73,6 +84,9 @@ class Build:
                             help='Select target to build')
         parser.add_argument('-c', '--cmd', '--task', dest='task',
                             help='Select which task should be executed')
+        parser.add_argument('--provenance',
+                            choices=['true', 'mode=min', 'mode=max'],
+                            help='Enable provenance attestation generation')
 
     def run(self, args):
         """
@@ -103,6 +117,21 @@ class BuildCommand(Command):
     def __str__(self):
         return 'build'
 
+    def _generate_attestation(self, ctx,
+                              time_started, time_finished,
+                              mode):
+        """
+            Generate the provenance attestation for the build.
+        """
+        predicate = Provenance(ctx, time_started, time_finished,
+                               mode)
+        stmt = Statement(predicate, ctx, time_started, time_finished).as_dict()
+        att_dir = Path(ctx.build_dir) / 'attestation'
+        att_dir.mkdir(parents=True, exist_ok=True)
+        with open(att_dir / 'kas-build.provenance.json', 'w') as f:
+            f.write(json.dumps(stmt, indent=4))
+            f.write('\n')
+
     def execute(self, ctx):
         """
             Executes the bitbake build command.
@@ -111,6 +140,8 @@ class BuildCommand(Command):
         bitbake = find_program(ctx.environ['PATH'], 'bitbake')
         cmd = [bitbake, '-c', ctx.config.get_bitbake_task()] \
             + self.extra_bitbake_args + ctx.config.get_bitbake_targets()
+
+        time_started = datetime.now()
         if sys.stdout.isatty():
             logging.info('%s$ %s', ctx.build_dir, ' '.join(cmd))
             ret = subprocess.call(cmd, env=ctx.environ, cwd=ctx.build_dir)
@@ -118,6 +149,12 @@ class BuildCommand(Command):
                 raise CommandExecError(cmd, ret)
         else:
             run_cmd(cmd, cwd=ctx.build_dir, liveupdate=True)
+        time_finished = datetime.now()
+
+        if ctx.args.provenance:
+            mode = Provenance.Mode.MAX if ctx.args.provenance == 'mode=max' \
+                else Provenance.Mode.MIN
+            self._generate_attestation(ctx, time_started, time_finished, mode)
 
 
 __KAS_PLUGINS__ = [Build]
diff -pruN 4.3.2-1/kas/plugins/dump.py 4.4-1/kas/plugins/dump.py
--- 4.3.2-1/kas/plugins/dump.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/kas/plugins/dump.py	2024-05-31 16:23:43.000000000 +0000
@@ -1,6 +1,6 @@
 # kas - setup tool for bitbake based projects
 #
-# Copyright (c) Siemens AG, 2017-2023
+# Copyright (c) Siemens AG, 2017-2024
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -36,6 +36,12 @@
     on the kas cmdline. For details on the locking support, see
     :class:`kas.includehandler.IncludeHandler`.
 
+    When running with ``--resolve-local``, VCS tracking information of the root
+    repo (the one with the kas-project.yml) is added to the output. The
+    generated file can be used as single input to kas to reproduce the build
+    environment. If the root repo is not under version control or contains
+    uncommitted changes, a warning is emitted.
+
     Please note:
 
     - the dumped config is semantically identical but not bit-by-bit identical
@@ -66,6 +72,7 @@
 import sys
 import json
 import yaml
+import logging
 from typing import TypeVar, TextIO
 from collections import OrderedDict
 from kas.context import get_context
@@ -162,6 +169,9 @@ class Dump(Checkout):
         parser.add_argument('--resolve-refs',
                             action='store_true',
                             help='Replace floating refs with exact SHAs')
+        parser.add_argument('--resolve-local',
+                            action='store_true',
+                            help='Add tracking information of root repo')
         lk_or_env.add_argument('--resolve-env',
                                action='store_true',
                                help='Set env defaults to captured env value')
@@ -190,6 +200,9 @@ class Dump(Checkout):
 
         if args.inplace and not args.lock:
             raise ArgsCombinationError('--inplace requires --lock')
+        if args.resolve_local and args.lock:
+            raise ArgsCombinationError(
+                '--resolve-local cannot be used with --lock')
 
         if args.lock:
             args.resolve_refs = True
@@ -207,12 +220,26 @@ class Dump(Checkout):
             del config_expanded['header']['includes']
 
         if args.resolve_refs and not args.lock:
-            for r in repos:
+            for r in filter(lambda r: not r.operations_disabled, repos):
                 if r.commit or r.branch or r.tag:
                     config_expanded['repos'][r.name]['commit'] = r.revision
                 elif r.refspec:
                     config_expanded['repos'][r.name]['refspec'] = r.revision
 
+        if args.resolve_local:
+            for r in filter(lambda r: r.operations_disabled and r.name, repos):
+                if r.revision:
+                    if r.dirty:
+                        logging.warning(f'Repository {r.name} (root repo) '
+                                        'contains uncommitted changes.')
+                    if config_expanded['repos'][r.name] is None:
+                        config_expanded['repos'][r.name] = {}
+                    config_expanded['repos'][r.name]['url'] = r.url
+                    config_expanded['repos'][r.name]['commit'] = r.revision
+                else:
+                    logging.warning(f'Repository {r.name} (root repo) '
+                                    'is not under version control.')
+
         if args.resolve_env and 'env' in config_expanded:
             config_expanded['env'] = ctx.config.get_environment()
 
diff -pruN 4.3.2-1/kas/plugins/for_all_repos.py 4.4-1/kas/plugins/for_all_repos.py
--- 4.3.2-1/kas/plugins/for_all_repos.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/kas/plugins/for_all_repos.py	2024-05-31 16:23:43.000000000 +0000
@@ -73,6 +73,7 @@ from kas.context import create_global_co
 from kas.config import Config
 from kas.libcmds import Macro, Command, SetupHome
 from kas.libkas import setup_parser_common_args
+from kas.libkas import setup_parser_keep_config_unchanged_arg
 from kas.libkas import setup_parser_preserve_env_arg
 from kas.libkas import run_handle_preserve_env_arg
 from kas.kasusererror import CommandExecError
@@ -91,6 +92,7 @@ class ForAllRepos:
     def setup_parser(cls, parser):
         setup_parser_common_args(parser)
         setup_parser_preserve_env_arg(parser)
+        setup_parser_keep_config_unchanged_arg(parser)
         parser.add_argument('command',
                             help='Command to be executed as a string.')
 
@@ -120,7 +122,8 @@ class ForAllReposCommand(Command):
                 'KAS_REPO_NAME': repo.name,
                 'KAS_REPO_PATH': repo.path,
                 'KAS_REPO_URL': '' if repo.operations_disabled else repo.url,
-                'KAS_REPO_COMMIT': repo.commit or '',
+                'KAS_REPO_COMMIT': '' if repo.operations_disabled
+                                   else (repo.commit or ''),
                 'KAS_REPO_BRANCH': repo.branch or '',
                 'KAS_REPO_TAG': repo.tag or '',
                 'KAS_REPO_REFSPEC': repo.refspec or '',
diff -pruN 4.3.2-1/kas/plugins/menu.py 4.4-1/kas/plugins/menu.py
--- 4.3.2-1/kas/plugins/menu.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/kas/plugins/menu.py	2024-05-31 16:23:43.000000000 +0000
@@ -79,7 +79,7 @@ from kas.repos import Repo
 from kas.includehandler import load_config as load_config_yaml, \
     SOURCE_DIR_OVERRIDE_KEY, SOURCE_DIR_HOST_OVERRIDE_KEY
 from kas.plugins.build import Build
-from kas.kasusererror import KasUserError
+from kas.kasusererror import KasUserError, MissingModuleError
 
 try:
     from snack import SnackScreen, EntryWindow, ButtonChoiceWindow, \
@@ -100,10 +100,6 @@ class VariableTypeError(KasUserError):
     pass
 
 
-class MissingModuleError(KasUserError):
-    pass
-
-
 class KConfigLoadError(KasUserError):
     """
     The KConfig file could not be found or is invalid
@@ -263,8 +259,7 @@ class Menu:
 
     def run(self, args):
         if not HAVE_NEWT:
-            raise MissingModuleError(
-                'Menu plugin requires \'python3-newt\' distribution package.')
+            raise MissingModuleError('python3-newt', 'Menu plugin')
 
         ctx = create_global_context(args)
 
@@ -298,6 +293,7 @@ class Menu:
             build_args.task = None
             build_args.extra_bitbake_args = []
             build_args.skip = None
+            build_args.provenance = False
 
             Build().run(build_args)
 
diff -pruN 4.3.2-1/kas/plugins/shell.py 4.4-1/kas/plugins/shell.py
--- 4.3.2-1/kas/plugins/shell.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/kas/plugins/shell.py	2024-05-31 16:23:43.000000000 +0000
@@ -44,6 +44,7 @@ from kas.context import create_global_co
 from kas.config import Config
 from kas.libcmds import Macro, Command, SetupHome
 from kas.libkas import setup_parser_common_args
+from kas.libkas import setup_parser_keep_config_unchanged_arg
 from kas.libkas import setup_parser_preserve_env_arg
 from kas.libkas import run_handle_preserve_env_arg
 from kas.kasusererror import CommandExecError
@@ -68,9 +69,7 @@ class Shell:
 
         setup_parser_common_args(parser)
         setup_parser_preserve_env_arg(parser)
-        parser.add_argument('-k', '--keep-config-unchanged',
-                            help='Skip steps that change the configuration',
-                            action='store_true')
+        setup_parser_keep_config_unchanged_arg(parser)
         parser.add_argument('-c', '--command',
                             help='Run command',
                             default='')
@@ -85,16 +84,6 @@ class Shell:
 
         run_handle_preserve_env_arg(ctx, os, args, SetupHome)
 
-        if args.keep_config_unchanged:
-            # Skip the tasks which would change the config of the build
-            # environment
-            args.skip += [
-                'setup_dir',
-                'finish_setup_repos',
-                'repos_apply_patches',
-                'write_bbconfig',
-            ]
-
         macro = Macro()
         macro.add(ShellCommand(args.command))
         macro.run(ctx, args.skip)
diff -pruN 4.3.2-1/kas/repos.py 4.4-1/kas/repos.py
--- 4.3.2-1/kas/repos.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/kas/repos.py	2024-05-31 16:23:43.000000000 +0000
@@ -90,6 +90,9 @@ class Repo:
         self._patches = patches
         self.operations_disabled = disable_operations
 
+        if not self.url:
+            self.resolve_local()
+
     @property
     def layers(self):
         return [os.path.join(self.path, layer).rstrip(os.sep + '.')
@@ -135,6 +138,14 @@ class Repo:
             return branch
         return None
 
+    @property
+    def dirty(self):
+        if not self.url:
+            return True
+        (_, output) = run_cmd(self.is_dirty_cmd(),
+                              cwd=self.path, fail=False)
+        return bool(output)
+
     def __str__(self):
         if self.commit and (self.tag or self.branch):
             refspec = f'{self.commit}({self.tag or self.branch})'
@@ -226,7 +237,6 @@ class Repo:
 
         if url is None:
             # No version control operation on repository
-            url = path
             disable_operations = True
 
         if repo_type == 'git':
@@ -352,10 +362,7 @@ class RepoImpl(Repo):
 
         if not get_context().force_checkout:
             # Check if repos is dirty
-            (_, output) = run_cmd(self.is_dirty_cmd(),
-                                  cwd=self.path,
-                                  fail=False)
-            if output:
+            if self.dirty:
                 logging.warning('Repo %s is dirty - no checkout', self.name)
                 return
 
@@ -387,7 +394,9 @@ class RepoImpl(Repo):
                 raise RepoRefError(
                     f'Branch "{self.branch}" cannot be found '
                     f'in repository "{self.name}"')
-            if self.commit:
+            # check if branch contains the requested commit.
+            # skip check on shallow clones, as branch information is missing
+            if self.commit and not get_context().repo_clone_depth:
                 (_, output) = run_cmd(self.branch_contains_ref(),
                                       cwd=self.path,
                                       fail=False)
@@ -483,12 +492,30 @@ class RepoImpl(Repo):
 
         return 0
 
+    def resolve_local(self):
+        (retc, output) = run_cmd(self.get_remote_url_cmd(),
+                                 cwd=self.path, fail=False)
+        if retc == 0:
+            self.url = output.strip()
+
+        (retc, output) = run_cmd(self.get_commit_cmd(),
+                                 cwd=self.path, fail=False)
+        if retc == 0:
+            self.commit = output.strip()
+        if self.url and self.commit:
+            logging.debug('Repository %s resolved to %s @ %s',
+                          self.name, self.url, self.commit)
+
 
 class GitRepo(RepoImpl):
     """
         Provides the git functionality for a Repo.
     """
 
+    @staticmethod
+    def get_type():
+        return 'git'
+
     def remove_ref_prefix(self, ref):
         ref_prefix = 'refs/'
         return ref[ref.startswith(ref_prefix) and len(ref_prefix):]
@@ -498,6 +525,19 @@ class GitRepo(RepoImpl):
 
     def clone_cmd(self, srcdir, createref):
         cmd = ['git', 'clone', '-q']
+
+        depth = get_context().repo_clone_depth
+        if depth:
+            if self.refspec:
+                logging.warning('Shallow cloning is not supported for legacy '
+                                f'refspec on repository "{self.name}". '
+                                'Performing full clone.')
+            else:
+                cmd.extend(['--depth', str(depth)])
+                if self.branch:
+                    cmd.extend(['--branch',
+                                self.remove_ref_prefix(self.branch)])
+
         if createref:
             cmd.extend([self.effective_url, '--bare', srcdir])
         elif srcdir:
@@ -518,11 +558,22 @@ class GitRepo(RepoImpl):
 
     def fetch_cmd(self):
         cmd = ['git', 'fetch', '-q']
+
+        depth = 0 if self.refspec else get_context().repo_clone_depth
+        if depth:
+            cmd.extend(['--depth', str(depth)])
+
         if self.tag:
-            cmd.append('--tags')
+            cmd.extend(['origin', f'+{self.tag}:refs/tags/{self.tag}'])
+            return cmd
+
+        # only fetch this commit (branch information is lost)
+        if depth and self.commit:
+            cmd.extend(['origin', self.commit])
+            return cmd
 
         branch = self.branch or self.refspec
-        if branch and branch.startswith('refs/'):
+        if branch and (branch.startswith('refs/') or depth):
             branch = self.remove_ref_prefix(branch)
             cmd.extend(['origin', f'+{branch}:refs/remotes/origin/{branch}'])
 
@@ -563,12 +614,22 @@ class GitRepo(RepoImpl):
     def set_remote_url_cmd(self):
         return ['git', 'remote', 'set-url', 'origin', self.effective_url]
 
+    def get_remote_url_cmd(self):
+        return ['git', 'remote', 'get-url', 'origin']
+
+    def get_commit_cmd(self):
+        return ['git', 'rev-parse', '--verify', 'HEAD']
+
 
 class MercurialRepo(RepoImpl):
     """
         Provides the hg functionality for a Repo.
     """
 
+    @staticmethod
+    def get_type():
+        return 'mercurial'
+
     def add_cmd(self):
         return ['hg', 'add']
 
@@ -620,3 +681,9 @@ class MercurialRepo(RepoImpl):
 
     def set_remote_url_cmd(self):
         raise NotImplementedError()
+
+    def get_remote_url_cmd(self):
+        return ['hg', 'paths', 'default']
+
+    def get_commit_cmd(self):
+        return ['hg', 'log', '-r', '.', '--template', '{node}\n']
diff -pruN 4.3.2-1/kas/schema-kas.json 4.4-1/kas/schema-kas.json
--- 4.3.2-1/kas/schema-kas.json	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/kas/schema-kas.json	2024-05-31 16:23:43.000000000 +0000
@@ -286,6 +286,12 @@
                 ]
             }
         },
+        "artifacts": {
+            "type": "object",
+            "additionalProperties": {
+                "type": ["string"]
+            }
+        },
         "_source_dir": {
             "type": "string"
         },
diff -pruN 4.3.2-1/kas-container 4.4-1/kas-container
--- 4.3.2-1/kas-container	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/kas-container	2024-05-31 16:23:43.000000000 +0000
@@ -2,7 +2,7 @@
 #
 # kas - setup tool for bitbake based projects
 #
-# Copyright (c) Siemens AG, 2018-2022
+# Copyright (c) Siemens AG, 2018-2024
 #
 # Authors:
 #  Jan Kiszka <jan.kiszka@siemens.com>
@@ -27,6 +27,11 @@
 
 set -e
 
+KAS_IMAGE_VERSION_DEFAULT="4.4"
+KAS_CONTAINER_IMAGE_PATH_DEFAULT="ghcr.io/siemens/kas"
+KAS_CONTAINER_IMAGE_NAME_DEFAULT="kas"
+KAS_CONTAINER_SELF_NAME="$(basename "$0")"
+
 usage()
 {
 	SELF="${KAS_CONTAINER_SELF_NAME}"
@@ -144,11 +149,6 @@ run_clean() {
 	fi
 }
 
-KAS_IMAGE_VERSION_DEFAULT="4.3.2"
-KAS_CONTAINER_IMAGE_PATH_DEFAULT="ghcr.io/siemens/kas"
-KAS_CONTAINER_IMAGE_NAME_DEFAULT="kas"
-KAS_CONTAINER_SELF_NAME="$(basename "$0")"
-
 set_container_image_var() {
 	KAS_IMAGE_VERSION="${KAS_IMAGE_VERSION:-${KAS_IMAGE_VERSION_DEFAULT}}"
 	KAS_CONTAINER_IMAGE_NAME="${KAS_CONTAINER_IMAGE_NAME:-${KAS_CONTAINER_IMAGE_NAME_DEFAULT}}"
@@ -330,15 +330,22 @@ while [ $# -gt 0 ] && [ $KAS_EXTRA_BITBA
 	case "$1" in
 	--skip|--target|--task)
 		KAS_OPTIONS="${KAS_OPTIONS} $1 $2"
-		shift 2
+		shift 1
+		shift 1 || KAS_OPTIONS="--help"
 		;;
 	-c|--cmd|--command)
 		KAS_BITBAKE_C_OPTION_ARGS="$2"
-		shift 2
+		shift 1
+		shift 1 || KAS_OPTIONS="--help"
 		;;
 	-E|--preserve-env)
 		fatal_error "$1 is not supported with ${KAS_CONTAINER_SELF_NAME}"
 		;;
+	--provenance)
+		KAS_OPTIONS="${KAS_OPTIONS} $1 $2"
+		shift 1
+		shift 1 || KAS_OPTIONS="--help"
+		;;
 	--)
 		KAS_EXTRA_BITBAKE_ARGS=$#
 		;;
@@ -539,7 +546,7 @@ if [ -n "${KAS_REPO_REF_DIR}" ]; then
 		-e KAS_REPO_REF_DIR=/repo-ref
 fi
 
-for var in TERM KAS_DISTRO KAS_MACHINE KAS_TARGET KAS_TASK \
+for var in TERM KAS_DISTRO KAS_MACHINE KAS_TARGET KAS_TASK KAS_CLONE_DEPTH \
            KAS_PREMIRRORS DISTRO_APT_PREMIRRORS BB_NUMBER_THREADS PARALLEL_MAKE \
            GIT_CREDENTIAL_USEHTTPPATH; do
 	if [ -n "$(eval echo \$${var})" ]; then
diff -pruN 4.3.2-1/kas-docker 4.4-1/kas-docker
--- 4.3.2-1/kas-docker	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/kas-docker	2024-05-31 16:23:43.000000000 +0000
@@ -2,7 +2,7 @@
 #
 # kas - setup tool for bitbake based projects
 #
-# Copyright (c) Siemens AG, 2018-2022
+# Copyright (c) Siemens AG, 2018-2024
 #
 # Authors:
 #  Jan Kiszka <jan.kiszka@siemens.com>
@@ -27,6 +27,11 @@
 
 set -e
 
+KAS_IMAGE_VERSION_DEFAULT="4.4"
+KAS_CONTAINER_IMAGE_PATH_DEFAULT="ghcr.io/siemens/kas"
+KAS_CONTAINER_IMAGE_NAME_DEFAULT="kas"
+KAS_CONTAINER_SELF_NAME="$(basename "$0")"
+
 usage()
 {
 	SELF="${KAS_CONTAINER_SELF_NAME}"
@@ -144,11 +149,6 @@ run_clean() {
 	fi
 }
 
-KAS_IMAGE_VERSION_DEFAULT="4.3.2"
-KAS_CONTAINER_IMAGE_PATH_DEFAULT="ghcr.io/siemens/kas"
-KAS_CONTAINER_IMAGE_NAME_DEFAULT="kas"
-KAS_CONTAINER_SELF_NAME="$(basename "$0")"
-
 set_container_image_var() {
 	KAS_IMAGE_VERSION="${KAS_IMAGE_VERSION:-${KAS_IMAGE_VERSION_DEFAULT}}"
 	KAS_CONTAINER_IMAGE_NAME="${KAS_CONTAINER_IMAGE_NAME:-${KAS_CONTAINER_IMAGE_NAME_DEFAULT}}"
@@ -330,15 +330,22 @@ while [ $# -gt 0 ] && [ $KAS_EXTRA_BITBA
 	case "$1" in
 	--skip|--target|--task)
 		KAS_OPTIONS="${KAS_OPTIONS} $1 $2"
-		shift 2
+		shift 1
+		shift 1 || KAS_OPTIONS="--help"
 		;;
 	-c|--cmd|--command)
 		KAS_BITBAKE_C_OPTION_ARGS="$2"
-		shift 2
+		shift 1
+		shift 1 || KAS_OPTIONS="--help"
 		;;
 	-E|--preserve-env)
 		fatal_error "$1 is not supported with ${KAS_CONTAINER_SELF_NAME}"
 		;;
+	--provenance)
+		KAS_OPTIONS="${KAS_OPTIONS} $1 $2"
+		shift 1
+		shift 1 || KAS_OPTIONS="--help"
+		;;
 	--)
 		KAS_EXTRA_BITBAKE_ARGS=$#
 		;;
@@ -539,7 +546,7 @@ if [ -n "${KAS_REPO_REF_DIR}" ]; then
 		-e KAS_REPO_REF_DIR=/repo-ref
 fi
 
-for var in TERM KAS_DISTRO KAS_MACHINE KAS_TARGET KAS_TASK \
+for var in TERM KAS_DISTRO KAS_MACHINE KAS_TARGET KAS_TASK KAS_CLONE_DEPTH \
            KAS_PREMIRRORS DISTRO_APT_PREMIRRORS BB_NUMBER_THREADS PARALLEL_MAKE \
            GIT_CREDENTIAL_USEHTTPPATH; do
 	if [ -n "$(eval echo \$${var})" ]; then
diff -pruN 4.3.2-1/scripts/lower-bound.py 4.4-1/scripts/lower-bound.py
--- 4.3.2-1/scripts/lower-bound.py	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/scripts/lower-bound.py	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,11 @@
+#!/usr/bin/env python3
+# takes a reverse-sorted, line separated list and
+# returns the first element that is equal or smaller
+# than the first argument
+
+import sys
+
+for line in sys.stdin:
+    if line.rstrip() <= sys.argv[1]:
+        print(line.rstrip())
+        break
diff -pruN 4.3.2-1/scripts/release.sh 4.4-1/scripts/release.sh
--- 4.3.2-1/scripts/release.sh	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/scripts/release.sh	2024-05-31 16:23:43.000000000 +0000
@@ -50,6 +50,7 @@ highlights=$(sed -e "/$OLD_VERSION$/,\$d
 prolog=$PWD/release-email.txt
 echo \
 "Hi all,
+!!! SET CONTAINER SHAs BEFORE SENDING !!!
 
 A new release $NEW_VERSION is available. A big thanks to all contributors:
 $authors
@@ -60,8 +61,11 @@ Thanks,
 Jan
 
 https://github.com/siemens/kas/releases/tag/$NEW_VERSION
+  ($(git rev-parse "$NEW_VERSION"))
 https://github.com/orgs/siemens/packages/container/package/kas%2Fkas
+  (ghcr.io/siemens/kas/kas:$NEW_VERSION@sha256:FILLME)
 https://github.com/orgs/siemens/packages/container/package/kas%2Fkas-isar
+  (ghcr.io/siemens/kas/kas-isar:$NEW_VERSION@sha256:FILLME)
 
 "> "$prolog"
 
diff -pruN 4.3.2-1/tests/conftest.py 4.4-1/tests/conftest.py
--- 4.3.2-1/tests/conftest.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/tests/conftest.py	2024-05-31 16:23:43.000000000 +0000
@@ -21,6 +21,7 @@
 # SOFTWARE.
 
 import pytest
+import os
 
 ENVVARS_KAS = [
     'KAS_WORK_DIR',
@@ -31,14 +32,38 @@ ENVVARS_KAS = [
     'KAS_TARGET',
     'KAS_TASK',
     'KAS_PREMIRRORS',
+    'KAS_CLONE_DEPTH',
     'SSH_PRIVATE_KEY',
     'SSH_PRIVATE_KEY_FILE',
     'SSH_AUTH_SOCK',
+    'CI_SERVER_HOST',
+    'CI_JOB_TOKEN',
+    'GITLAB_CI',
+    'GITHUB_ACTIONS'
+]
+
+ENVVARS_TOOLS = [
+    'EMAIL'
 ]
 
 
 @pytest.fixture
-def monkeykas(monkeypatch):
-    for var in ENVVARS_KAS:
+def monkeykas(monkeypatch, tmpdir):
+    for var in ENVVARS_KAS + ENVVARS_TOOLS:
         monkeypatch.delenv(var, raising=False)
+    # Set HOME to a temporary directory
+    homedir = tmpdir / '_home'
+    homedir.mkdir()
+    monkeypatch.setenv('HOME', str(homedir))
+
+    # remove all git related variables
+    for var in os.environ.keys():
+        if var.startswith('GIT_'):
+            monkeypatch.delenv(var)
+    # provide minimal git environment
+    monkeypatch.setenv('GIT_AUTHOR_NAME', 'kas')
+    monkeypatch.setenv('GIT_AUTHOR_EMAIL', 'kas@example.com')
+    monkeypatch.setenv('GIT_COMMITTER_NAME', 'kas')
+    monkeypatch.setenv('GIT_COMMITTER_EMAIL', 'kas@example.com')
+
     yield monkeypatch
diff -pruN 4.3.2-1/tests/test_build/artifact-glob.yml 4.4-1/tests/test_build/artifact-glob.yml
--- 4.3.2-1/tests/test_build/artifact-glob.yml	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/tests/test_build/artifact-glob.yml	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,8 @@
+header:
+  version: 17
+
+repos:
+  this:
+
+artifacts:
+  disk-file: bitbake.opt*
diff -pruN 4.3.2-1/tests/test_build/artifact-invalid.yml 4.4-1/tests/test_build/artifact-invalid.yml
--- 4.3.2-1/tests/test_build/artifact-invalid.yml	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/tests/test_build/artifact-invalid.yml	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,8 @@
+header:
+  version: 17
+
+repos:
+  this:
+
+artifacts:
+  disk-file: does-not-exist
diff -pruN 4.3.2-1/tests/test_build/artifact-named.yml 4.4-1/tests/test_build/artifact-named.yml
--- 4.3.2-1/tests/test_build/artifact-named.yml	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/tests/test_build/artifact-named.yml	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,8 @@
+header:
+  version: 17
+
+repos:
+  this:
+
+artifacts:
+  disk-file: bitbake.options
diff -pruN 4.3.2-1/tests/test_build/bitbake 4.4-1/tests/test_build/bitbake
--- 4.3.2-1/tests/test_build/bitbake	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/tests/test_build/bitbake	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,3 @@
+#!/bin/sh
+
+echo "$@" > bitbake.options
diff -pruN 4.3.2-1/tests/test_build/oe-init-build-env 4.4-1/tests/test_build/oe-init-build-env
--- 4.3.2-1/tests/test_build/oe-init-build-env	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/tests/test_build/oe-init-build-env	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,3 @@
+#!/bin/sh
+
+export PATH=$(pwd):$PATH
diff -pruN 4.3.2-1/tests/test_build/provenance.yml 4.4-1/tests/test_build/provenance.yml
--- 4.3.2-1/tests/test_build/provenance.yml	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/tests/test_build/provenance.yml	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,15 @@
+header:
+  version: 17
+
+env:
+  CAPTURE_THIS: null
+
+repos:
+  this:
+
+  kas:
+    url: https://github.com/siemens/kas.git
+    commit: 907816a5c4094b59a36aec12226e71c461c05b77
+
+artifacts:
+  disk-file: bitbake.options
diff -pruN 4.3.2-1/tests/test_build.py 4.4-1/tests/test_build.py
--- 4.3.2-1/tests/test_build.py	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/tests/test_build.py	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,66 @@
+# kas - setup tool for bitbake based projects
+#
+# Copyright (c) Siemens AG, 2024
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be
+# included in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
+
+import shutil
+import pytest
+import json
+from kas import kas
+from kas.kasusererror import ArtifactNotFoundError
+
+BITBAKE_OPTIONS_SHA256 = "e35d535e81cfdc4ed304af8000284c36" \
+                         "19d2c4c78392ddcefe9ca46b158235f8"
+
+
+def test_artifact_node(monkeykas, tmpdir):
+    tdir = str(tmpdir / 'test_build')
+    shutil.copytree('tests/test_build', tdir)
+    monkeykas.chdir(tdir)
+    kas.kas(['build', 'artifact-named.yml'])
+    kas.kas(['build', 'artifact-glob.yml'])
+    kas.kas(['build', 'artifact-invalid.yml'])
+
+
+def test_provenance(monkeykas, tmpdir):
+    tdir = str(tmpdir / 'test_build')
+    shutil.copytree('tests/test_build', tdir)
+    monkeykas.chdir(tdir)
+
+    with pytest.raises(ArtifactNotFoundError):
+        kas.kas(['build', '--provenance', 'mode=min',
+                 'artifact-invalid.yml'])
+
+    kas.kas(['build', '--provenance', 'mode=min', 'provenance.yml'])
+    with open('build/attestation/kas-build.provenance.json', 'r') as f:
+        prov = json.load(f)
+        assert prov['subject'][0]['name'] == 'bitbake.options'
+        assert 'env' not in \
+            prov['predicate']['buildDefinition']['internalParameters']
+
+    with monkeykas.context() as mp:
+        mp.setenv('CAPTURE_THIS', 'OK Sir!')
+        kas.kas(['build', '--provenance', 'mode=max', 'provenance.yml'])
+    with open('build/attestation/kas-build.provenance.json', 'r') as f:
+        prov = json.load(f)
+        params = prov['predicate']['buildDefinition']['internalParameters']
+        assert params['env']['CAPTURE_THIS'] == 'OK Sir!'
+        assert prov['subject'][0]['name'] == 'bitbake.options'
+        assert prov['subject'][0]['digest']['sha256'] == BITBAKE_OPTIONS_SHA256
diff -pruN 4.3.2-1/tests/test_commands/test-local-hg.yml 4.4-1/tests/test_commands/test-local-hg.yml
--- 4.3.2-1/tests/test_commands/test-local-hg.yml	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/tests/test_commands/test-local-hg.yml	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,6 @@
+header:
+  version: 15
+
+repos:
+  local:
+    type: hg
diff -pruN 4.3.2-1/tests/test_commands/test-local.yml 4.4-1/tests/test_commands/test-local.yml
--- 4.3.2-1/tests/test_commands/test-local.yml	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/tests/test_commands/test-local.yml	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,5 @@
+header:
+  version: 15
+
+repos:
+  local:
diff -pruN 4.3.2-1/tests/test_commands/test-shallow.yml 4.4-1/tests/test_commands/test-shallow.yml
--- 4.3.2-1/tests/test_commands/test-shallow.yml	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/tests/test_commands/test-shallow.yml	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,23 @@
+header:
+  version: 14
+
+repos:
+  this:
+
+  kas_1:
+    url: https://github.com/siemens/kas.git
+    branch: master
+
+  kas_2:
+    url: https://github.com/siemens/kas.git
+    tag: '4.3'
+    commit: f650ebe2495a9cbe2fdf4a2c8becc7b3db470d55
+
+  kas_3:
+    url: https://github.com/siemens/kas.git
+    commit: e42a64a666082b77fbc2758b07191b662d17f792
+
+  kas_4:
+    url: https://github.com/siemens/kas.git
+    # keep legacy refspec here for testing purposes
+    refspec: master
diff -pruN 4.3.2-1/tests/test_commands/test-url-rewrite.yml 4.4-1/tests/test_commands/test-url-rewrite.yml
--- 4.3.2-1/tests/test_commands/test-url-rewrite.yml	1970-01-01 00:00:00.000000000 +0000
+++ 4.4-1/tests/test_commands/test-url-rewrite.yml	2024-05-31 16:23:43.000000000 +0000
@@ -0,0 +1,9 @@
+header:
+  version: 14
+
+repos:
+  this:
+
+  kas:
+    url: git@github.com:siemens/kas.git
+    commit: 907816a5c4094b59a36aec12226e71c461c05b77
diff -pruN 4.3.2-1/tests/test_commands.py 4.4-1/tests/test_commands.py
--- 4.3.2-1/tests/test_commands.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/tests/test_commands.py	2024-05-31 16:23:43.000000000 +0000
@@ -26,9 +26,11 @@ import pathlib
 import shutil
 import json
 import yaml
+import subprocess
 import pytest
 from kas import kas
-from kas.libkas import TaskExecError
+from kas.libkas import run_cmd
+from kas.libkas import TaskExecError, KasUserError
 
 
 def test_for_all_repos(monkeykas, tmpdir):
@@ -47,6 +49,19 @@ def test_for_all_repos(monkeykas, tmpdir
             == 'e9ca55a239caa1a2098e1d48773a29ea53c6cab2'
 
 
+def test_for_all_repos_keep_config_unchanged(monkeykas, tmpdir):
+    tdir = str(tmpdir / 'test_commands')
+    shutil.copytree('tests/test_commands', tdir)
+    monkeykas.chdir(tdir)
+
+    with pytest.raises(FileNotFoundError, match=r'.*/kas_1.[01]'):
+        kas.kas(['for-all-repos', '--keep-config-unchanged', 'test.yml',
+                 'pwd'])
+
+    assert not os.path.exists('kas_1.0')
+    assert not os.path.exists("kas_1.1")
+
+
 def test_checkout(monkeykas, tmpdir):
     tdir = str(tmpdir / 'test_commands')
     shutil.copytree('tests/test_commands', tdir)
@@ -71,6 +86,17 @@ def test_invalid_checkout(monkeykas, tmp
         kas.kas(['checkout', 'test-invalid.yml'])
 
 
+def test_checkout_with_ci_rewrite(monkeykas, tmpdir):
+    tdir = str(tmpdir / 'test_commands')
+    shutil.copytree('tests/test_commands', tdir)
+    monkeykas.chdir(tdir)
+    with monkeykas.context() as mp:
+        mp.setenv('GITLAB_CI', 'true')
+        mp.setenv('CI_SERVER_HOST', 'github.com')
+        mp.setenv('CI_JOB_TOKEN', 'not-needed')
+        kas.kas(['checkout', 'test-url-rewrite.yml'])
+
+
 def test_checkout_create_refs(monkeykas, tmpdir):
     tdir = str(tmpdir / 'test_commands')
     repo_cache = pathlib.Path(str(tmpdir.mkdir('repos')))
@@ -82,6 +108,68 @@ def test_checkout_create_refs(monkeykas,
     assert os.path.exists('kas/.git/objects/info/alternates')
 
 
+def test_checkout_shallow(monkeykas, tmpdir):
+    tdir = str(tmpdir / 'test_commands')
+    shutil.copytree('tests/test_commands', tdir)
+    monkeykas.chdir(tdir)
+    with monkeykas.context() as mp:
+        mp.setenv('KAS_CLONE_DEPTH', 'invalid')
+        with pytest.raises(KasUserError):
+            kas.kas(['checkout', 'test-shallow.yml'])
+
+    with monkeykas.context() as mp:
+        mp.setenv('KAS_CLONE_DEPTH', '1')
+        kas.kas(['checkout', 'test-shallow.yml'])
+    for repo in ['kas_1', 'kas_2', 'kas_3', 'kas_4']:
+        (rc, output) = run_cmd(['git', 'rev-list', '--count', 'HEAD'],
+                               cwd=repo, fail=False, liveupdate=False)
+        assert rc == 0
+        if repo == 'kas_4':
+            assert output.strip() >= '1'
+        else:
+            assert output.strip() == '1'
+
+
+def test_shallow_updates(monkeykas, tmpdir):
+    def _get_commit(repo):
+        (rc, output) = run_cmd(['git', 'rev-parse', '--verify', 'HEAD'],
+                               cwd=repo, fail=False, liveupdate=False)
+        assert rc == 0
+        return output.strip()
+
+    tdir = tmpdir / 'test_commands'
+    tdir.mkdir()
+    shutil.copy('tests/test_commands/oe-init-build-env', tdir)
+    monkeykas.chdir(tdir)
+    monkeykas.setenv('KAS_CLONE_DEPTH', '1')
+    # test non-pinned checkout of master branch
+    base_yml = {'header': {'version': 15}, 'repos': {
+                'this': {},
+                'kas': {
+                    'url': 'https://github.com/siemens/kas.git',
+                    'branch': 'master'
+                }}}
+    with open(tdir / 'kas.yml', 'w') as f:
+        yaml.dump(base_yml, f)
+    kas.kas(['checkout', 'kas.yml'])
+    # switch branches, perform checkout again
+    base_yml['repos']['kas']['branch'] = 'next'
+    with open(tdir / 'kas.yml', 'w') as f:
+        yaml.dump(base_yml, f)
+    kas.kas(['checkout', 'kas.yml'])
+    # pin commit on next branch
+    commit = '5d1ab6e8ed3a12c7093c9041f104fb6a2db701a1'
+    base_yml_lock = {'header': {'version': 15},
+                     'overrides': {'repos': {'kas': {'commit': commit}}}}
+    with open(tdir / 'kas.lock.yml', 'w') as f:
+        yaml.dump(base_yml_lock, f)
+    kas.kas(['checkout', 'kas.yml'])
+    assert _get_commit('kas') == commit
+    # update to latest revision of next branch
+    kas.kas(['checkout', '--update', 'kas.yml'])
+    assert _get_commit('kas') != commit
+
+
 def test_repo_includes(monkeykas, tmpdir):
     tdir = str(tmpdir / 'test_commands')
     shutil.copytree('tests/test_repo_includes', tdir)
@@ -171,3 +259,84 @@ def test_lockfile(monkeykas, tmpdir, cap
         lockspec = yaml.safe_load(f)
         assert lockspec['overrides']['repos']['externalrepo']['commit'] \
             != test_commit
+
+
+def test_root_resolve_novcs(monkeykas, tmpdir, capsys):
+    tdir = str(tmpdir.mkdir('test_root_resolve_novcs'))
+    shutil.rmtree(tdir, ignore_errors=True)
+    shutil.copytree('tests/test_commands', tdir)
+    monkeykas.chdir(tdir)
+
+    capsys.readouterr()
+    kas.kas('dump --resolve-local test-local.yml'.split())
+    console = capsys.readouterr()
+    assert 'not under version control' in console.err
+    data = yaml.safe_load(console.out)
+    assert data['repos']['local'] is None
+
+
+def test_root_resolve_git(monkeykas, tmpdir, capsys):
+    tdir = str(tmpdir.mkdir('test_root_resolve_git'))
+    shutil.rmtree(tdir, ignore_errors=True)
+    shutil.copytree('tests/test_commands', tdir)
+    monkeykas.chdir(tdir)
+
+    upstream_url = 'http://github.com/siemens/kas.git'
+    subprocess.check_call(['git', 'init'])
+    subprocess.check_call(['git', 'branch', '-m', 'main'])
+    subprocess.check_call(['git', 'add', 'oe-init-build-env', '*.yml'])
+    subprocess.check_call(['git', 'commit', '-m', 'test'])
+    subprocess.check_call(['git', 'remote', 'add', 'origin', upstream_url])
+    commit = subprocess.check_output(['git', 'rev-parse', '--verify', 'HEAD'])
+
+    capsys.readouterr()
+    kas.kas('dump --resolve-local test-local.yml'.split())
+    console = capsys.readouterr()
+    assert 'contains uncommitted' not in console.err
+    data = yaml.safe_load(console.out)
+    assert data['repos']['local']['commit'] == commit.decode('utf-8').strip()
+    assert data['repos']['local']['url'] == upstream_url
+
+    # make repository dirty
+    with open(f'{tdir}/new-file.txt', 'w') as f:
+        f.write('test')
+    kas.kas('dump --resolve-local test-local.yml'.split())
+    console = capsys.readouterr()
+    assert 'contains uncommitted' in console.err
+    data = yaml.safe_load(console.out)
+    assert data['repos']['local']['commit'] == commit.decode('utf-8').strip()
+    assert data['repos']['local']['url'] == upstream_url
+
+
+def test_root_resolve_hg(monkeykas, tmpdir, capsys):
+    tdir = str(tmpdir.mkdir('test_root_resolve_hg'))
+    shutil.rmtree(tdir, ignore_errors=True)
+    shutil.copytree('tests/test_commands', tdir)
+    monkeykas.chdir(tdir)
+
+    upstream_url = 'http://github.com/siemens/kas'
+    subprocess.check_call(['hg', 'init'])
+    subprocess.check_call(['hg', 'add', 'test-local-hg.yml',
+                           'oe-init-build-env'])
+    subprocess.check_call(['hg', 'commit', '-m', 'test'])
+    with open(f'{tdir}/.hg/hgrc', mode='w') as f:
+        f.write(f'[paths]\ndefault = {upstream_url}')
+    commit = subprocess.check_output(['hg', 'log', '-r', '.',
+                                      '--template', '{node}\n'])
+
+    capsys.readouterr()
+    kas.kas('dump --resolve-local test-local-hg.yml'.split())
+    console = capsys.readouterr()
+    assert 'contains uncommitted' in console.err
+    data = yaml.safe_load(console.out)
+    assert data['repos']['local']['commit'] == commit.decode('utf-8').strip()
+    assert data['repos']['local']['url'] == upstream_url
+    assert data['repos']['local']['type'] == 'hg'
+
+    # add all files to repository
+    subprocess.check_call(['hg', 'add'])
+    subprocess.check_call(['hg', 'commit', '-m', 'test2'])
+    capsys.readouterr()
+    kas.kas('dump --resolve-local test-local-hg.yml'.split())
+    console = capsys.readouterr()
+    assert 'contains uncommitted' not in console.err
diff -pruN 4.3.2-1/tests/test_environment_variables.py 4.4-1/tests/test_environment_variables.py
--- 4.3.2-1/tests/test_environment_variables.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/tests/test_environment_variables.py	2024-05-31 16:23:43.000000000 +0000
@@ -115,6 +115,7 @@ def _test_env_section_export(monkeykas,
 
     shutil.copytree('tests/test_environment_variables', str(conf_dir))
     monkeykas.chdir(conf_dir)
+    monkeykas.setenv('KAS_CLONE_DEPTH', '1')
 
     # Overwrite oe-init-build-env script
     # BB_ENV_* filter variables are only exported by
diff -pruN 4.3.2-1/tests/test_refspec/test10.yml 4.4-1/tests/test_refspec/test10.yml
--- 4.3.2-1/tests/test_refspec/test10.yml	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/tests/test_refspec/test10.yml	2024-05-31 16:23:43.000000000 +0000
@@ -4,8 +4,7 @@ header:
 repos:
   this:
 
-  evolve:
-    url: https://repo.mercurial-scm.org/evolve/
-    type: hg
-    branch: stable
-    commit: '6634:991cbf0f66f2'
+  kas:
+    url: https://github.com/siemens/kas.git
+    tag: 3.0.1
+    branch: master
diff -pruN 4.3.2-1/tests/test_refspec/test11.yml 4.4-1/tests/test_refspec/test11.yml
--- 4.3.2-1/tests/test_refspec/test11.yml	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/tests/test_refspec/test11.yml	1970-01-01 00:00:00.000000000 +0000
@@ -1,10 +0,0 @@
-header:
-  version: 15
-
-repos:
-  this:
-
-  kas:
-    url: https://github.com/siemens/kas.git
-    tag: 3.0.1
-    branch: master
diff -pruN 4.3.2-1/tests/test_refspec/test9.yml 4.4-1/tests/test_refspec/test9.yml
--- 4.3.2-1/tests/test_refspec/test9.yml	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/tests/test_refspec/test9.yml	2024-05-31 16:23:43.000000000 +0000
@@ -4,7 +4,8 @@ header:
 repos:
   this:
 
-  bitbake:
-    url: https://github.com/openembedded/bitbake.git
-    branch: '2.2'
-    commit: 191e6eb2bceb467c97e315301f1f64722cf0e976
+  evolve:
+    url: https://repo.mercurial-scm.org/evolve/
+    type: hg
+    branch: stable
+    commit: '6634:991cbf0f66f2'
diff -pruN 4.3.2-1/tests/test_refspec.py 4.4-1/tests/test_refspec.py
--- 4.3.2-1/tests/test_refspec.py	2024-04-09 17:25:46.000000000 +0000
+++ 4.4-1/tests/test_refspec.py	2024-05-31 16:23:43.000000000 +0000
@@ -225,7 +225,7 @@ def test_branch_and_tag(monkeykas, tmpdi
     shutil.copytree('tests/test_refspec', tdir)
     monkeykas.chdir(tdir)
     with pytest.raises(RepoRefError):
-        kas.kas(['checkout', 'test10.yml'])
+        kas.kas(['checkout', 'test9.yml'])
 
     with pytest.raises(RepoRefError):
-        kas.kas(['checkout', 'test11.yml'])
+        kas.kas(['checkout', 'test10.yml'])
